//**********************************************************************
// This source code is provided as an example.  No warranty is expressed
// or implied.  It is your responsibility to determine its suitability
// to your application, to modify it as necessary, and to validate
// its correctness.
//
// Licensed Materials - Property of IBM
// (c) Copyright IBM Corporation 2000. All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication, or
// disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//**********************************************************************
// Revision / History
// Defect     Date       Who      Description
// ---------- 01/03/2000 ldbader  Module created
//**********************************************************************

package com.ibm.etill.ldbcardcassette;

import  java.sql.ResultSet;
import  java.sql.SQLException;
import  java.util.Enumeration;
import  java.util.Hashtable;
import  java.util.Vector;

import  com.ibm.etill.framework.archive.ETillArchive;
import  com.ibm.etill.framework.cassette.query.CassetteQuery;
import  com.ibm.etill.framework.cassette.query.CassetteExtensionObject;
import  com.ibm.etill.framework.log.ErrorLog;
import  com.ibm.etill.framework.log.Trace;
import  com.ibm.etill.framework.payapi.BatchKey;
import  com.ibm.etill.framework.payapi.ETillAbortOperation;
import  com.ibm.etill.framework.payapi.OrderKey;
import  com.ibm.etill.framework.supervisor.AccountKey;
import  com.ibm.etill.framework.supervisor.CreditKey;
import  com.ibm.etill.framework.supervisor.PaymentKey;
import  com.ibm.etill.framework.xdm.AccountQueryRequest;
import  com.ibm.etill.framework.xdm.BatchQueryRequest;
import  com.ibm.etill.framework.xdm.CreditQueryRequest;
import  com.ibm.etill.framework.xdm.OrderQueryRequest;
import  com.ibm.etill.framework.xdm.PaymentQueryRequest;
import  com.ibm.etill.framework.xdm.PSServerAccount;
import  com.ibm.etill.framework.xdm.PSServerBatch;
import  com.ibm.etill.framework.xdm.PSServerCassetteConstants;
import  com.ibm.etill.framework.xdm.PSServerCredit;
import  com.ibm.etill.framework.xdm.PSServerCreditConstants;
import  com.ibm.etill.framework.xdm.PSServerOrder;
import  com.ibm.etill.framework.xdm.PSServerOrderConstants;
import  com.ibm.etill.framework.xdm.PSServerPayment;
import  com.ibm.etill.framework.xdm.PSServerPaymentConstants;
import  com.ibm.etill.framework.xdm.QueryRequest;

/***********************************************************************
* The cassette query object is responsible for processing query
* requests from the Payment Manager user interface.  It is constructed
* in the framework's servlet during initialization and used whenever
* a query request needs to return cassette dependent information for
* the cassette extentions to framework objects.
* <p>
* For this cassette, the cassette has an extension for
* <ul>
* <li> orders
* <li> payments
* <li> credits
* <li> batches
* <li> accounts.
* </ul>
* <p>
* Query requests for these objects will be processed.  No processing
* will be done for any unrecognized objects.
* <p>
* For each query request, cassette dependent data will be selected
* from the cassette dependent table and the appropriate XML descriptors
* will be added to the matching framework object's XML descriptor.
***********************************************************************/
public final class LdbCardCassetteQuery
   extends CassetteQuery
   implements LdbCardConstants {

   private static final String COPYRIGHT =
      "(c) Copyright IBM Corporation 2000 - All Rights Reserved.";

   //<gutter>
   //</gutter>

// /********************************************************************
// * This static block initializes the trace settings for this class.
// * By default, it turns on all the possible traces.  This is
// * necessary because the current version of the servlet does not
// * provide a way for the operator to set cassette dependent trace
// * values.
// ********************************************************************/
   static {
      Trace.setTraceSettings(TRACE_ID, Trace.TRACE_ALL);
   }

   /********************************************************************
   * This constructor is used by the framework to create the
   * cassette query object for this cassette.  Since the class is
   * loaded dynamically, the no argument constructor is always used.
   * <p>
   * For this cassette, the object simply initializes its parent object
   * with the payment type constant used for this cassette.
   ********************************************************************/
   public LdbCardCassetteQuery() {
      super(PAYMENT_TYPE);

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardCassetteQuery.LdbCardCassetteQuery()"
         );
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardCassetteQuery.LdbCardCassetteQuery()"
         );
      }
   }

   /********************************************************************
   * This method is used by the framework when performing a query
   * operation that includes framework objects that could be
   * extended by this cassette.
   * <p>
   * For this cassette, the request is used to initialize an appropriate
   * inner query object suitable for extracting the cassette dependent
   * information from the cassette dependent tables.  After obtaining
   * the information, this object is asked to combine it with the
   * information already gathered for the framework objects.
   *
   * @param     argRequest - a query request to be serviced.
   * @param     argFrameworkObjects - a vector of XML information
   * objects for the framework objects that match the request's
   * selection criteria.
   * @exception ETillAbortOperation - any error results in this
   * exception.
   ********************************************************************/
   public void query (
      QueryRequest           argRequest,
      Vector                 argFrameworkObjects
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardCassetteQuery.query()"
         );
      }

      // Determine what type of request was passed.  Then build the
      // appropriate cassette query and combine its results with
      // the framework results.  Note that any unrecognized
      // request will simply be ignored.
      if (argRequest instanceof OrderQueryRequest) {
         OrderQuery varQuery = new OrderQuery(
            (OrderQueryRequest)argRequest
         );
         varQuery.combine(argFrameworkObjects);
      } else if (argRequest instanceof PaymentQueryRequest) {
         PaymentQuery varQuery = new PaymentQuery(
            (PaymentQueryRequest)argRequest
         );
         varQuery.combine(argFrameworkObjects);
      } else if (argRequest instanceof CreditQueryRequest) {
         CreditQuery varQuery = new CreditQuery(
            (CreditQueryRequest)argRequest
         );
         varQuery.combine(argFrameworkObjects);
      } else if (argRequest instanceof BatchQueryRequest) {
         BatchQuery varQuery = new BatchQuery(
            (BatchQueryRequest)argRequest
         );
         varQuery.combine(argFrameworkObjects);
      } else if (argRequest instanceof AccountQueryRequest) {
         AccountQuery varQuery = new AccountQuery(
            (AccountQueryRequest)argRequest
         );
         varQuery.combine(argFrameworkObjects);
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardCassetteQuery.query()"
         );
      }
   }

   /********************************************************************
   * An order query object automatically selects information from
   * the cassette dependent order table using the same criteria
   * that was used to select information from the framework order
   * table.  It is responsible for building the correct SQL statement,
   * performing the query, and collecting the results.  Furthermore,
   * it is responsible for combining the information from each
   * cassette order extension with the information already gathered
   * for the framework order.
   * <p>
   * A query object trades simplicity for efficiency.  The simplest
   * technique for acquiring cassette dependent information for
   * each order would be to perform a SQL Select statement for
   * each framework order passed to the cassette query object.
   * Instead, this query object performs a single SQL Select statement,
   * joining the cassette dependent order table with the framework
   * order table using the same criteria used to select the framework
   * orders passed to the cassette query object.
   ********************************************************************/
   public final class OrderQuery extends QueryRequest {

      //<gutter>
      //</gutter>

      /*****************************************************************
      * This query attribute is a collection of all the information
      * selected from the cassette dependent order table.  The key
      * is constructed using the table's primary key columns and the
      * value is an inner order information object.
      *****************************************************************/
      private Hashtable objOrders = new Hashtable();

      /*****************************************************************
      * This constructor is use by the cassette query object when it
      * is asked to service an OrderQuery request. During construction,
      * it obtains and collects the cassette dependent order information
      * using same criteria that was used to obtain the framework order
      * information.
      *
      * @param     argRequest - the OrderQuery request.
      * @exception ETillAbortOperation - any error results in this
      * exception.
      *****************************************************************/
      public OrderQuery (
         OrderQueryRequest argRequest
      ) throws ETillAbortOperation {
         super(argRequest);
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.OrderQuery()"
            );
         }

         this.query();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.OrderQuery()"
            );
         }
      }

      /*****************************************************************
      * This method is used by this object to select the appropriate
      * columns for the query.
      * <p>
      * For this cassette, all columns are selected.
      *
      * @return - a String that lists the columns to be included in
      * the query's result set.
      *****************************************************************/
      protected String selectStatement() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.selectStatement()"
            );
         }

         String varResult = "*";

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.selectStatement()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to identify the name of
      * the table to be used in the query.
      * <p>
      * For this cassette, the table name is the name of a cassette
      * dependent view that can be found in the database.
      *
      * @return - a String containing the owner qualified table name.
      *****************************************************************/
      protected String tableName() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.tableName()"
            );
         }

         String varResult =
            ETillArchive.geteTillDatabaseOwner() +
            "." +
            VIEW_ORDERS;

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.tableName()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the From
      * clause for the SQL Select statement to be performed.  There
      * are three cases:
      * <ol>
      * <li>The merchant has issued a query for orders and provided
      * order selection criteria so this query will obtain order
      * information for the orders selected
      * <li>The merchant has issued a query for payments and provided
      * payment selection criteria so this query will obtain order
      * information for the payments selected
      * <li>The merchant has issued a query for credits and provided
      * credit selection criteria so this query will obtain order
      * information for the credits selected.
      * </ol>
      * <p>
      * For this cassette, the correct From clause is built depending on
      * the criteria supplied by the original request.
      *
      * @return - The string containing the From clause to be used
      * in the query
      *****************************************************************/
      protected String fromClause() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.fromClause()"
            );
         }

         // By default, assume that the original request selected
         // orders.  The cassette dependent order view can be obtained
         // using the same criteria.  If this is not the case, the
         // result will be replaced below.
         String varResult = this.tableName();
         // Obtain the Where clause from the original request.  If
         // the where clause is null, no selection criteria was
         // specified by the merchant.
         QueryRequest varOriginalRequest = this.getOriginalRequest();
         String varOriginalWhere = varOriginalRequest.whereCondition();
         if (varOriginalWhere == null) {
            varOriginalWhere = "";
         }
         // Determine what type of selection criteria was specified
         // by examining the type of the original request.
         if (varOriginalRequest instanceof PaymentQueryRequest) {
            // In this case, the original request selected payments
            // so build a Where clause by joining the rows of the
            // cassette dependent order table with the selected
            // rows from the framework payment table.
            varResult =
               " (SELECT DISTINCT " +
                  PSServerOrderConstants.VIEW_MERCHANTNAME + ", " +
                  PSServerOrderConstants.VIEW_ORDERNUMBER  +
               " FROM " +
                  ETillArchive.geteTillDatabaseOwner() + "." +
                  PSServerPaymentConstants.PAYMENTVIEW +
               " WHERE " + varOriginalWhere + ") M , " +
               this.tableName() +
               " WHERE " +
                  this.tableName() + "." + VIEW_MERCHANT_NAME +
                  " = M." + PSServerOrderConstants.VIEW_MERCHANTNAME +
                  " AND " +
                  this.tableName() + "." + VIEW_ORDER +
                  " = M." + PSServerOrderConstants.VIEW_ORDERNUMBER;
         } else if (varOriginalRequest instanceof CreditQueryRequest) {
            // In this case, the original request selected credits
            // so build a Where clause by joining the rows of the
            // cassette dependent order table with the selected
            // rows from the framework credit table.
            varResult =
               " (SELECT DISTINCT " +
                  PSServerOrderConstants.VIEW_MERCHANTNAME + ", " +
                  PSServerOrderConstants.VIEW_ORDERNUMBER  +
               " FROM " +
                  ETillArchive.geteTillDatabaseOwner() + "." +
                  PSServerCreditConstants.CREDITVIEW +
               " WHERE " + varOriginalWhere + ") M , " +
               this.tableName() +
               " WHERE " +
                  this.tableName() + "." + VIEW_MERCHANT_NAME +
                  " = M." + PSServerOrderConstants.VIEW_MERCHANTNAME +
                  " AND " +
                  this.tableName() + "." + VIEW_ORDER +
                  " = M." + PSServerOrderConstants.VIEW_ORDERNUMBER;
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.fromClause()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the Where
      * clause for the SQL Select statement to be performed.  There
      * are three cases:
      * <ol>
      * <li>The merchant has issued a query for orders and provided
      * order selection criteria so this query will obtain order
      * information for the orders selected
      * <li>The merchant has issued a query for payments and provided
      * payment selection criteria so this query will obtain order
      * information for the payments selected
      * <li>The merchant has issued a query for credits and provided
      * credit selection criteria so this query will obtain order
      * information for the credits selected.
      * </ol>
      * <p>
      * For this cassette, the correct Where clause is built depending
      * on the criteria supplied by the original request.
      *
      * @return - The string containing the Where clause to be used
      * in the query
      *****************************************************************/
      public String whereCondition() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.whereCondition()"
            );
         }

         // By default, assume that the original request selected
         // orders.  The cassette dependent order view can be obtained
         // using the same criteria.  If this is not the case, the
         // Where clause has already been incorporated into the
         // subtable selected in the From clause and the outer
         // where clause can be null.
         QueryRequest varOriginalRequest = this.getOriginalRequest();
         String varResult = varOriginalRequest.whereCondition();
         if (varOriginalRequest instanceof PaymentQueryRequest) {
            varResult = null;
         } else if (varOriginalRequest instanceof CreditQueryRequest) {
            varResult = null;
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.whereCondition()"
            );
         }
         return varResult;
      }

      /****************************************************************
      * This method is used by this class to process the results of
      * the SQL Select statement.  For each row in the result set,
      * it constructs an inner order information object and saves it
      * in a collection using the proper order key.
      *
      * @param     argResultSet - The result set produced when the
      * manufactured SQL Select statement was performed.
      * @return - An integer containing the number of cassette
      * dependent order information objects in the collection.
      * @exception ETillAbortOperation - Any error results in this
      * exception.
      *****************************************************************/
      protected int processQueryResults(
         ResultSet argResultSet
      ) throws ETillAbortOperation {

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.processQueryResults()"
            );
         }

         try {
            while (argResultSet.next()) {
               OrderInfo varOrderInfo = new OrderInfo(argResultSet);
               this.objOrders.put(
                  varOrderInfo.getKey(),
                  varOrderInfo
               );
            }
         } catch (SQLException varException) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "OrderQuery SQLException caught." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_QUERY_ORD_SELECT_SQL_FAILURE
               );
            }
            ErrorLog.logError(
               RB_ID,
               MSG_QUERY_ORD_SELECT_SQL_FAILURE,
               varException
            );
            throw new ETillAbortOperation(
               PRC_DATABASE_ERROR,
               SRC_QUERY_ORD_SELECT_SQL_FAILURE
            );
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.processQueryResults()"
            );
         }
         return this.objOrders.size();
      }

      /*****************************************************************
      * This method is used by the cassette query object to combine
      * cassette dependent order information with framework order
      * information. For each framework order object in the collection
      * provided, it determines if there is a matching cassette order
      * information object and, if so, asks the cassette order
      * information object to add its XML fields to the result.
      *
      * @param     argFrameworkObjects - a collection of framework
      * order objects previously selected by the framework while
      * processing the merchant's query request.
      *****************************************************************/
      public void combine(
         Vector argFrameworkObjects
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " OrderQuery.combine()"
            );
         }
         Enumeration varOrderList = argFrameworkObjects.elements();
         while (varOrderList.hasMoreElements()) {
            PSServerOrder varFrameworkOrderInfo =
               (PSServerOrder)varOrderList.nextElement();
            OrderInfo varCassetteOrderInfo =
               (OrderInfo)this.objOrders.get(
                  new OrderKey(
                     varFrameworkOrderInfo.getMerchantNumber(),
                     varFrameworkOrderInfo.getOrderNumber()
                  )
               );
            if (varCassetteOrderInfo != null) {
               varCassetteOrderInfo.combine(
                  varFrameworkOrderInfo
               );
            }

         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " OrderQuery.combine()"
            );
         }
      }

      /*****************************************************************
      * An order information object contains the information gathered
      * from one row of the cassette dependent order table.  It is
      * responsible for providing a hashtable key for the referended
      * order and for combining its information with matching
      * framework order information.
      *****************************************************************/
      public final class OrderInfo {

         //<gutter>
         //</gutter>

         private String objMerchant        = null;
         private String objOrder           = null;
         private String objAmountApproved  = null;
         private String objAccountNumber   = null;
         private String objPan             = null;
         private String objExpiry          = null;
         private String objBrand           = null;
         private String objAddress         = null;
         private String objPostalCode      = null;

         /**************************************************************
         * This constructor is used by the order query to save the
         * information selected from a row in the cassette dependent
         * order table.
         *
         * @param     argResultSet - a ResultSet where the current row
         * contains the information to be recorded in this object.
         * @exception SQLException - if an error occurs while reading
         * the columns from the row, this execption is thrown.
         **************************************************************/
         public OrderInfo(
            ResultSet argResultSet
         ) throws SQLException {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " OrderInfo.OrderInfo()"
               );
            }
            this.objMerchant =
               ETillArchive.readString(argResultSet, VIEW_MERCHANT_NAME);
            this.objOrder =
               ETillArchive.readString(argResultSet, VIEW_ORDER);
            this.objAmountApproved =
               ETillArchive.readString(argResultSet, VIEW_AMOUNT);
            this.objAccountNumber =
               ETillArchive.readString(argResultSet, VIEW_ACCOUNT);
            this.objPan =
               ETillArchive.readString(argResultSet, VIEW_PAN);
            this.objExpiry =
               ETillArchive.readString(argResultSet, VIEW_EXPIRY);
            this.objBrand =
               ETillArchive.readString(argResultSet, VIEW_BRAND);
            this.objAddress =
               ETillArchive.readString(argResultSet, VIEW_ADDRESS);
            this.objPostalCode =
               ETillArchive.readString(argResultSet, VIEW_POSTAL_CODE);

            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " OrderInfo.OrderInfo()"
               );
            }
         }

         /**************************************************************
         * This method is used by the order query to obtain a key
         * that will uniquely identify this object in a hashtable.
         *
         * @return - a framework order key that uniquely identifies
         * this object.
         **************************************************************/
         public OrderKey getKey() {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " OrderInfo.getKey()"
               );
            }

            OrderKey varOrderKey = new OrderKey (
               this.objMerchant,
               this.objOrder
            );


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " OrderInfo.getKey()"
               );
            }
            return varOrderKey;
         }

         /**************************************************************
         * This method is used by the order query to combine the XML
         * descriptors for this object with the XML descriptors for
         * the matching framework order.
         *
         * @param     argFrameworkOrderInfo - the XML descriptors for
         * the matching framework order.
         **************************************************************/
         public void combine (
            PSServerOrder argFrameworkOrderInfo
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " OrderInfo.combine()"
               );
            }

            CassetteExtensionObject varOrderXml =
               new CassetteExtensionObject();
            varOrderXml.addProperty(XDM_AMOUNT,this.objAmountApproved,null);
            varOrderXml.addProperty(XDM_ACCOUNT,this.objAccountNumber,null);
            varOrderXml.addProperty(XDM_PAN,this.objPan,null);
            varOrderXml.addProperty(XDM_EXPIRY,this.objExpiry,null);
            varOrderXml.addProperty(XDM_BRAND,this.objBrand,null);
            varOrderXml.addProperty(XDM_ADDRESS,this.objAddress,null);
            varOrderXml.addProperty(XDM_POSTAL_CODE,this.objPostalCode,null);

            argFrameworkOrderInfo.setCassetteExtensionObject(varOrderXml);


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " OrderInfo.combine()"
               );
            }
         }

      }

   }

   /********************************************************************
   * A payment query object automatically selects information from
   * the cassette dependent payment table using the same criteria
   * that was used to select information from the framework payment
   * table.  It is responsible for building the correct SQL statement,
   * performing the query, and collecting the results.  Furthermore,
   * it is responsible for combining the information from each
   * cassette payment extension with the information already gathered
   * for the framework payment.
   * <p>
   * A query object trades simplicity for efficiency.  The simplest
   * technique for acquiring cassette dependent information for
   * each payment would be to perform a SQL Select statement for
   * each framework payment passed to the cassette query object.
   * Instead, this query object performs a single SQL Select statement,
   * joining the cassette dependent payment table with the framework
   * payment table using the same criteria used to select the framework
   * payments passed to the cassette query object.
   ********************************************************************/
   public final class PaymentQuery extends QueryRequest {

      //<gutter>
      //</gutter>

      /*****************************************************************
      * This query attribute is a collection of all the information
      * selected from the cassette dependent payment table.  The key
      * is constructed using the table's primary key columns and the
      * value is an inner payment information object.
      *****************************************************************/
      private Hashtable objPayments = new Hashtable();

      /*****************************************************************
      * This constructor is use by the cassette query object when it
      * is asked to service a PaymentQuery request. During construction,
      * it obtains and collects the cassette dependent payment
      * information using same criteria that was used to obtain the
      * framework payment information.
      *
      * @param     argRequest - the PaymentQuery request.
      * @exception ETillAbortOperation - any error results in this
      * exception.
      *****************************************************************/
      public PaymentQuery (
         PaymentQueryRequest argRequest
      ) throws ETillAbortOperation {
         super(argRequest);
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.PaymentQuery()"
            );
         }

         this.query();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.PaymentQuery()"
            );
         }
      }

      /*****************************************************************
      * This method is used by this object to select the appropriate
      * columns for the query.
      * <p>
      * For this cassette, all columns are selected.
      *
      * @return - a String that lists the columns to be included in
      * the query's result set.
      *****************************************************************/
      protected String selectStatement() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.selectStatement()"
            );
         }

         String varResult = "*";

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.selectStatement()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to identify the name of
      * the table to be used in the query. There are two cases:
      * <ol>
      * <li>The merchant has issued a query for payments and provided
      * payment selection criteria so this query will obtain payment
      * information for the payments selected
      * <li>The merchant has issued a query for payments within a batch
      * and provided batch selection criteria so this query will obtain
      * payment information for the batches selected
      * </ol>
      * <p>
      * <p>
      * For this cassette, the table name is the name of a cassette
      * dependent view that can be found in the database.
      *
      * @return - a String containing the owner qualified table name.
      *****************************************************************/
      protected String tableName() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.tableName()"
            );
         }

         String varResult =
            ETillArchive.geteTillDatabaseOwner() +
            "." +
            VIEW_PAYMENTS;

         QueryRequest varOriginalRequest = this.getOriginalRequest();
         if (varOriginalRequest instanceof BatchQueryRequest) {
            varResult =
               ETillArchive.geteTillDatabaseOwner() +
               "." +
               VIEW_BATCH_PAYMENTS;
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.tableName()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the From
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the method returns the payment view selected
      * by this object's tableName() method.
      *
      * @return - The string containing the From clause to be used
      * in the query
      *****************************************************************/
      protected String fromClause() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.fromClause()"
            );
         }

         String varResult = this.tableName();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.fromClause()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the Where
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the payment view selected by this object's
      * tableName() method is designed to use the same Where clause
      * specified in the original request to select the framework
      * payment objects.
      *
      * @return - The string containing the Where clause to be used
      * in the query
      *****************************************************************/
      public String whereCondition() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.whereCondition()"
            );
         }

         QueryRequest varOriginalRequest = this.getOriginalRequest();
         String varResult = varOriginalRequest.whereCondition();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.whereCondition()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this class to process the results of
      * the SQL Select statement.  For each row in the result set,
      * it constructs an inner payment information object and saves it
      * in a collection using the proper payment key.
      *
      * @param     argResultSet - The result set produced when the
      * manufactured SQL Select statement was performed.
      * @return - An integer containing the number of cassette
      * dependent payment information objects in the collection.
      * @exception ETillAbortOperation - Any error results in this
      * exception.
      *****************************************************************/
      protected int processQueryResults(
         ResultSet argResultSet
      ) throws ETillAbortOperation {

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.processQueryResults()"
            );
         }

         try {
            while (argResultSet.next()) {
               PaymentInfo varPaymentInfo = new PaymentInfo(argResultSet);
               this.objPayments.put(
                  varPaymentInfo.getKey(),
                  varPaymentInfo
               );
            }
         } catch (SQLException varException) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "PaymentQuery SQLException caught." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_QUERY_PAY_SELECT_SQL_FAILURE
               );
            }
            ErrorLog.logError(
               RB_ID,
               MSG_QUERY_PAY_SELECT_SQL_FAILURE,
               varException
            );
            throw new ETillAbortOperation(
               PRC_DATABASE_ERROR,
               SRC_QUERY_PAY_SELECT_SQL_FAILURE
            );
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.processQueryResults()"
            );
         }
         return this.objPayments.size();
      }

      /*****************************************************************
      * This method is used by the cassette query object to combine
      * cassette dependent payment information with framework payment
      * information. For each framework payment object in the collection
      * provided, it determines if there is a matching cassette payment
      * information object and, if so, asks the cassette payment
      * information object to add its XML fields to the result.
      *
      * @param     argFrameworkObjects - a collection of framework
      * payment objects previously selected by the framework while
      * processing the merchant's query request.
      *****************************************************************/
      public void combine(
         Vector argFrameworkObjects
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " PaymentQuery.combine()"
            );
         }
         Enumeration varPaymentList = argFrameworkObjects.elements();
         while (varPaymentList.hasMoreElements()) {
            PSServerPayment varFrameworkPaymentInfo =
               (PSServerPayment)varPaymentList.nextElement();
            PaymentInfo varCassettePaymentInfo =
               (PaymentInfo)this.objPayments.get(
                  new PaymentKey(
                     varFrameworkPaymentInfo.getMerchantNumber(),
                     varFrameworkPaymentInfo.getOrderNumber(),
                     varFrameworkPaymentInfo.getPaymentNumber()
                  )
               );
            if (varCassettePaymentInfo != null) {
               varCassettePaymentInfo.combine(
                  varFrameworkPaymentInfo
               );
            }

         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " PaymentQuery.combine()"
            );
         }
      }

      /*****************************************************************
      * A payment information object contains the information gathered
      * from one row of the cassette dependent payment table.  It is
      * responsible for providing a hashtable key for the referenced
      * payment and for combining its information with matching
      * framework payment information.
      *****************************************************************/
      public final class PaymentInfo {

         //<gutter>
         //</gutter>

         private String objMerchant        = null;
         private String objOrder           = null;
         private String objPayment         = null;
         private String objApproval        = null;
         private String objReason          = null;

         /**************************************************************
         * This constructor is used by the payment query to save the
         * information selected from a row in the cassette dependent
         * payment table.
         *
         * @param     argResultSet - a ResultSet where the current row
         * contains the information to be recorded in this object.
         * @exception SQLException - if an error occurs while reading
         * the columns from the row, this execption is thrown.
         **************************************************************/
         public PaymentInfo(
            ResultSet argResultSet
         ) throws SQLException {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " PaymentInfo.PaymentInfo()"
               );
            }
            this.objMerchant =
               ETillArchive.readString(argResultSet, VIEW_MERCHANT_NAME);
            this.objOrder =
               ETillArchive.readString(argResultSet, VIEW_ORDER);
            this.objPayment =
               ETillArchive.readString(argResultSet, VIEW_PAYMENT);
            this.objApproval =
               ETillArchive.readString(argResultSet, VIEW_APPROVAL);
            this.objReason =
               ETillArchive.readString(argResultSet, VIEW_REASON);

            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " PaymentInfo.PaymentInfo()"
               );
            }
         }

         /**************************************************************
         * This method is used by the payment query to obtain a key
         * that will uniquely identify this object in a hashtable.
         *
         * @return - a framework payment key that uniquely identifies
         * this object.
         **************************************************************/
         public PaymentKey getKey() {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " PaymentInfo.getKey()"
               );
            }

            PaymentKey varPaymentKey = new PaymentKey (
               this.objMerchant,
               this.objOrder,
               this.objPayment
            );


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " PaymentInfo.getKey()"
               );
            }
            return varPaymentKey;
         }

         /**************************************************************
         * This method is used by the payment query to combine the XML
         * descriptors for this object with the XML descriptors for
         * the matching framework payment.
         *
         * @param     argFrameworkPaymentInfo - the XML descriptors for
         * the matching framework payment.
         **************************************************************/
         public void combine (
            PSServerPayment argFrameworkPaymentInfo
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " PaymentInfo.combine()"
               );
            }

            CassetteExtensionObject varPaymentXml =
               new CassetteExtensionObject();
            varPaymentXml.addProperty(XDM_APPROVAL,this.objApproval,null);
            varPaymentXml.addProperty(XDM_REASON,this.objReason,null);

            argFrameworkPaymentInfo.setCassetteExtensionObject(varPaymentXml);


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " PaymentInfo.combine()"
               );
            }
         }

      }

   }

   /********************************************************************
   * A credit query object automatically selects information from
   * the cassette dependent credit table using the same criteria
   * that was used to select information from the framework credit
   * table.  It is responsible for building the correct SQL statement,
   * performing the query, and collecting the results.  Furthermore,
   * it is responsible for combining the information from each
   * cassette credit extension with the information already gathered
   * for the framework credit.
   * <p>
   * A query object trades simplicity for efficiency.  The simplest
   * technique for acquiring cassette dependent information for
   * each credit would be to perform a SQL Select statement for
   * each framework credit passed to the cassette query object.
   * Instead, this query object performs a single SQL Select statement,
   * joining the cassette dependent credit table with the framework
   * credit table using the same criteria used to select the framework
   * credits passed to the cassette query object.
   ********************************************************************/
   public final class CreditQuery extends QueryRequest {

      //<gutter>
      //</gutter>

      /*****************************************************************
      * This query attribute is a collection of all the information
      * selected from the cassette dependent credit table.  The key
      * is constructed using the table's primary key columns and the
      * value is an inner credit information object.
      *****************************************************************/
      private Hashtable objCredits = new Hashtable();

      /*****************************************************************
      * This constructor is use by the cassette query object when it
      * is asked to service a CreditQuery request. During construction,
      * it obtains and collects the cassette dependent credit
      * information using same criteria that was used to obtain the
      * framework credit information.
      *
      * @param     argRequest - the PaymentQuery request.
      * @exception ETillAbortOperation - any error results in this
      * exception.
      *****************************************************************/
      public CreditQuery (
         CreditQueryRequest argRequest
      ) throws ETillAbortOperation {
         super(argRequest);
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.CreditQuery()"
            );
         }

         this.query();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.CreditQuery()"
            );
         }
      }

      /*****************************************************************
      * This method is used by this object to select the appropriate
      * columns for the query.
      * <p>
      * For this cassette, all columns are selected.
      *
      * @return - a String that lists the columns to be included in
      * the query's result set.
      *****************************************************************/
      protected String selectStatement() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.selectStatement()"
            );
         }

         String varResult = "*";

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.selectStatement()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to identify the name of
      * the table to be used in the query.  There are two cases:
      * <ol>
      * <li>The merchant has issued a query for payments and provided
      * payment selection criteria so this query will obtain payment
      * information for the payments selected
      * <li>The merchant has issued a query for payments within a batch
      * and provided batch selection criteria so this query will obtain
      * payment information for the batches selected
      * </ol>
      * <p>
      * <p>
      * For this cassette, the table name is the name of a cassette
      * dependent view that can be found in the database.
      *
      * @return - a String containing the owner qualified table name.
      *****************************************************************/
      protected String tableName() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.tableName()"
            );
         }

         String varResult =
            ETillArchive.geteTillDatabaseOwner() +
            "." +
            VIEW_CREDITS;

         QueryRequest varOriginalRequest = this.getOriginalRequest();
         if (varOriginalRequest instanceof BatchQueryRequest) {
            varResult =
               ETillArchive.geteTillDatabaseOwner() +
               "." +
               VIEW_BATCH_CREDITS;
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.tableName()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the From
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the method returns the credit view selected
      * by this object's tableName() method.
      *
      * @return - The string containing the From clause to be used
      * in the query
      *****************************************************************/
      protected String fromClause() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.fromClause()"
            );
         }

         String varResult = this.tableName();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.fromClause()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the Where
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the credit view selected by this object's
      * tableName() method is designed to use the same Where clause
      * specified in the original request to select the framework
      * credit objects.
      *
      * @return - The string containing the Where clause to be used
      * in the query
      *****************************************************************/
      public String whereCondition() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.whereCondition()"
            );
         }

         QueryRequest varOriginalRequest = this.getOriginalRequest();
         String varResult = varOriginalRequest.whereCondition();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.whereCondition()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this class to process the results of
      * the SQL Select statement.  For each row in the result set,
      * it constructs an inner credit information object and saves it
      * in a collection using the proper credit key.
      *
      * @param     argResultSet - The result set produced when the
      * manufactured SQL Select statement was performed.
      * @return - An integer containing the number of cassette
      * dependent credit information objects in the collection.
      * @exception ETillAbortOperation - Any error results in this
      * exception.
      *****************************************************************/
      protected int processQueryResults(
         ResultSet argResultSet
      ) throws ETillAbortOperation {

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.processQueryResults()"
            );
         }

         try {
            while (argResultSet.next()) {
               CreditInfo varCreditInfo = new CreditInfo(argResultSet);
               this.objCredits.put(
                  varCreditInfo.getKey(),
                  varCreditInfo
               );
            }
         } catch (SQLException varException) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "CreditQuery SQLException caught." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_QUERY_CRE_SELECT_SQL_FAILURE
               );
            }
            ErrorLog.logError(
               RB_ID,
               MSG_QUERY_CRE_SELECT_SQL_FAILURE,
               varException
            );
            throw new ETillAbortOperation(
               PRC_DATABASE_ERROR,
               SRC_QUERY_CRE_SELECT_SQL_FAILURE
            );
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.processQueryResults()"
            );
         }
         return this.objCredits.size();
      }

      /*****************************************************************
      * This method is used by the cassette query object to combine
      * cassette dependent credit information with framework credit
      * information. For each framework credit object in the collection
      * provided, it determines if there is a matching cassette credit
      * information object and, if so, asks the cassette credit
      * information object to add its XML fields to the result.
      *
      * @param     argFrameworkObjects - a collection of framework
      * credit objects previously selected by the framework while
      * processing the merchant's query request.
      *****************************************************************/
      public void combine(
         Vector argFrameworkObjects
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " CreditQuery.combine()"
            );
         }
         Enumeration varCreditList = argFrameworkObjects.elements();
         while (varCreditList.hasMoreElements()) {
            PSServerCredit varFrameworkCreditInfo =
               (PSServerCredit)varCreditList.nextElement();
            CreditInfo varCassetteCreditInfo =
               (CreditInfo)this.objCredits.get(
                  new CreditKey(
                     varFrameworkCreditInfo.getMerchantNumber(),
                     varFrameworkCreditInfo.getOrderNumber(),
                     varFrameworkCreditInfo.getCreditNumber()
                  )
               );
            if (varCassetteCreditInfo != null) {
               varCassetteCreditInfo.combine(
                  varFrameworkCreditInfo
               );
            }

         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " CreditQuery.combine()"
            );
         }
      }

      /*****************************************************************
      * A credit information object contains the information gathered
      * from one row of the cassette dependent credit table.  It is
      * responsible for providing a hashtable key for the referenced
      * credit and for combining its information with matching
      * framework credit information.
      *****************************************************************/
      public final class CreditInfo {

         //<gutter>
         //</gutter>

         private String objMerchant        = null;
         private String objOrder           = null;
         private String objCredit          = null;
         private String objApproval        = null;
         private String objReason          = null;

         /**************************************************************
         * This constructor is used by the credit query to save the
         * information selected from a row in the cassette dependent
         * credit table.
         *
         * @param     argResultSet - a ResultSet where the current row
         * contains the information to be recorded in this object.
         * @exception SQLException - if an error occurs while reading
         * the columns from the row, this execption is thrown.
         **************************************************************/
         public CreditInfo(
            ResultSet argResultSet
         ) throws SQLException {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " CreditInfo.CreditInfo()"
               );
            }
            this.objMerchant =
               ETillArchive.readString(argResultSet, VIEW_MERCHANT_NAME);
            this.objOrder =
               ETillArchive.readString(argResultSet, VIEW_ORDER);
            this.objCredit =
               ETillArchive.readString(argResultSet, VIEW_CREDIT);
            this.objApproval =
               ETillArchive.readString(argResultSet, VIEW_APPROVAL);
            this.objReason =
               ETillArchive.readString(argResultSet, VIEW_REASON);

            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " CreditInfo.CreditInfo()"
               );
            }
         }

         /**************************************************************
         * This method is used by the credit query to obtain a key
         * that will uniquely identify this object in a hashtable.
         *
         * @return - a framework credit key that uniquely identifies
         * this object.
         **************************************************************/
         public CreditKey getKey() {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " CreditInfo.getKey()"
               );
            }

            CreditKey varCreditKey = new CreditKey (
               this.objMerchant,
               this.objOrder,
               this.objCredit
            );


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " CreditInfo.getKey()"
               );
            }
            return varCreditKey;
         }

         /**************************************************************
         * This method is used by the credit query to combine the XML
         * descriptors for this object with the XML descriptors for
         * the matching framework credit.
         *
         * @param     argFrameworkCreditInfo - the XML descriptors for
         * the matching framework credit.
         **************************************************************/
         public void combine (
            PSServerCredit argFrameworkCreditInfo
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " CreditInfo.combine()"
               );
            }

            CassetteExtensionObject varCreditXml =
               new CassetteExtensionObject();
            varCreditXml.addProperty(XDM_APPROVAL,this.objApproval,null);
            varCreditXml.addProperty(XDM_REASON,this.objReason,null);

            argFrameworkCreditInfo.setCassetteExtensionObject(varCreditXml);


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " CreditInfo.combine()"
               );
            }
         }

      }

   }

   /********************************************************************
   * A batch query object automatically selects information from
   * the cassette dependent batch table using the same criteria
   * that was used to select information from the framework batch
   * table.  It is responsible for building the correct SQL statement,
   * performing the query, and collecting the results.  Furthermore,
   * it is responsible for combining the information from each
   * cassette batch extension with the information already gathered
   * for the framework batch.
   * <p>
   * A query object trades simplicity for efficiency.  The simplest
   * technique for acquiring cassette dependent information for
   * each batch would be to perform a SQL Select statement for
   * each framework batch passed to the cassette query object.
   * Instead, this query object performs a single SQL Select statement,
   * joining the cassette dependent batch table with the framework
   * batch table using the same criteria used to select the framework
   * batches passed to the cassette query object.
   ********************************************************************/
   public final class BatchQuery extends QueryRequest {

      //<gutter>
      //</gutter>

      /*****************************************************************
      * This query attribute is a collection of all the information
      * selected from the cassette dependent batch table.  The key
      * is constructed using the table's primary key columns and the
      * value is an inner batch information object.
      *****************************************************************/
      private Hashtable objBatches = new Hashtable();

      /*****************************************************************
      * This constructor is use by the cassette query object when it
      * is asked to service a BatchQuery request. During construction,
      * it obtains and collects the cassette dependent batch
      * information using same criteria that was used to obtain the
      * framework batch information.
      *
      * @param     argRequest - the BatchQuery request.
      * @exception ETillAbortOperation - any error results in this
      * exception.
      *****************************************************************/
      public BatchQuery (
         BatchQueryRequest argRequest
      ) throws ETillAbortOperation {
         super(argRequest);
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.BatchQuery()"
            );
         }

         this.query();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.BatchQuery()"
            );
         }
      }

      /*****************************************************************
      * This method is used by this object to select the appropriate
      * columns for the query.
      * <p>
      * For this cassette, all columns are selected.
      *
      * @return - a String that lists the columns to be included in
      * the query's result set.
      *****************************************************************/
      protected String selectStatement() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.selectStatement()"
            );
         }

         String varResult = "*";

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.selectStatement()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to identify the name of
      * the table to be used in the query.
      * <p>
      * For this cassette, the table name is the name of a cassette
      * dependent view that can be found in the database.
      *
      * @return - a String containing the owner qualified table name.
      *****************************************************************/
      protected String tableName() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.tableName()"
            );
         }

         String varResult =
            ETillArchive.geteTillDatabaseOwner() +
            "." +
            VIEW_BATCHES;

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.tableName()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the From
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the method returns the batch view selected
      * by this object's tableName() method.
      *
      * @return - The string containing the From clause to be used
      * in the query
      *****************************************************************/
      protected String fromClause() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.fromClause()"
            );
         }

         String varResult = this.tableName();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.fromClause()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the Where
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the batch view selected by this object's
      * tableName() method is designed to use the same Where clause
      * specified in the original request to select the framework
      * batch objects.
      *
      * @return - The string containing the Where clause to be used
      * in the query
      *****************************************************************/
      public String whereCondition() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.whereCondition()"
            );
         }

         QueryRequest varOriginalRequest = this.getOriginalRequest();
         String varResult = varOriginalRequest.whereCondition();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.whereCondition()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this class to process the results of
      * the SQL Select statement.  For each row in the result set,
      * it constructs an inner batch information object and saves it
      * in a collection using the proper batch key.
      *
      * @param     argResultSet - The result set produced when the
      * manufactured SQL Select statement was performed.
      * @return - An integer containing the number of cassette
      * dependent batch information objects in the collection.
      * @exception ETillAbortOperation - Any error results in this
      * exception.
      *****************************************************************/
      protected int processQueryResults(
         ResultSet argResultSet
      ) throws ETillAbortOperation {

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.processQueryResults()"
            );
         }

         try {
            while (argResultSet.next()) {
               BatchInfo varBatchInfo = new BatchInfo(argResultSet);
               this.objBatches.put(
                  varBatchInfo.getKey(),
                  varBatchInfo
               );
            }
         } catch (SQLException varException) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "BatchQuery SQLException caught." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_QUERY_BAT_SELECT_SQL_FAILURE
               );
            }
            ErrorLog.logError(
               RB_ID,
               MSG_QUERY_BAT_SELECT_SQL_FAILURE,
               varException
            );
            throw new ETillAbortOperation(
               PRC_DATABASE_ERROR,
               SRC_QUERY_BAT_SELECT_SQL_FAILURE
            );
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.processQueryResults()"
            );
         }
         return this.objBatches.size();
      }

      /*****************************************************************
      * This method is used by the cassette query object to combine
      * cassette dependent batch information with framework batch
      * information. For each framework batch object in the collection
      * provided, it determines if there is a matching cassette batch
      * information object and, if so, asks the cassette batch
      * information object to add its XML fields to the result.
      *
      * @param     argFrameworkObjects - a collection of framework
      * credit objects previously selected by the framework while
      * processing the merchant's query request.
      *****************************************************************/
      public void combine(
         Vector argFrameworkObjects
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " BatchQuery.combine()"
            );
         }
         Enumeration varBatchList = argFrameworkObjects.elements();
         while (varBatchList.hasMoreElements()) {
            PSServerBatch varFrameworkBatchInfo =
               (PSServerBatch)varBatchList.nextElement();
            BatchInfo varCassetteBatchInfo =
               (BatchInfo)this.objBatches.get(
                  new BatchKey(
                     varFrameworkBatchInfo.getMerchantNumber(),
                     varFrameworkBatchInfo.getBatchNumber()
                  )
               );
            if (varCassetteBatchInfo != null) {
               varCassetteBatchInfo.combine(
                  varFrameworkBatchInfo
               );
            }

         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " BatchQuery.combine()"
            );
         }
      }

      /*****************************************************************
      * A batch information object contains the information gathered
      * from one row of the cassette dependent batch table.  It is
      * responsible for providing a hashtable key for the referenced
      * batch and for combining its information with matching
      * framework batch information.
      *****************************************************************/
      public final class BatchInfo {

         //<gutter>
         //</gutter>

         private String objMerchant        = null;
         private String objBatch           = null;
         private String objApproval        = null;
         private String objReason          = null;
         private String objClosedToTrans   = null;

         /**************************************************************
         * This constructor is used by the batch query to save the
         * information selected from a row in the cassette dependent
         * batch table.
         *
         * @param     argResultSet - a ResultSet where the current row
         * contains the information to be recorded in this object.
         * @exception SQLException - if an error occurs while reading
         * the columns from the row, this execption is thrown.
         **************************************************************/
         public BatchInfo(
            ResultSet argResultSet
         ) throws SQLException {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " BatchInfo.BatchInfo()"
               );
            }
            this.objMerchant =
               ETillArchive.readString(argResultSet, VIEW_MERCHANT_NAME);
            this.objBatch =
               ETillArchive.readString(argResultSet, VIEW_BATCH);
            this.objApproval =
               ETillArchive.readString(argResultSet, VIEW_APPROVAL);
            this.objReason =
               ETillArchive.readString(argResultSet, VIEW_REASON);
            this.objClosedToTrans =
               ETillArchive.readString(argResultSet, VIEW_CLOSED_TO_TRANS);

            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " BatchInfo.BatchInfo()"
               );
            }
         }

         /**************************************************************
         * This method is used by the batch query to obtain a key
         * that will uniquely identify this object in a hashtable.
         *
         * @return - a framework batch key that uniquely identifies
         * this object.
         **************************************************************/
         public BatchKey getKey() {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " BatchInfo.getKey()"
               );
            }

            BatchKey varBatchKey = new BatchKey (
               this.objMerchant,
               this.objBatch
            );


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " BatchInfo.getKey()"
               );
            }
            return varBatchKey;
         }

         /**************************************************************
         * This method is used by the batch query to combine the XML
         * descriptors for this object with the XML descriptors for
         * the matching framework batch.
         *
         * @param     argFrameworkBatchInfo - the XML descriptors for
         * the matching framework batch.
         **************************************************************/
         public void combine (
            PSServerBatch argFrameworkBatchInfo
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " BatchInfo.combine()"
               );
            }

            CassetteExtensionObject varBatchXml =
               new CassetteExtensionObject();
            varBatchXml.addProperty(
               XDM_APPROVAL,
               this.objApproval,
               null
            );
            varBatchXml.addProperty(
               XDM_REASON,
               this.objReason,
               null
            );
            varBatchXml.addProperty(
               XDM_CLOSED_TO_TRANS,
               this.objClosedToTrans,
               null
            );

            argFrameworkBatchInfo.setCassetteExtensionObject(varBatchXml);


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " BatchInfo.combine()"
               );
            }
         }

      }

   }

   /********************************************************************
   * An account query object automatically selects information from
   * the cassette dependent account table using the same criteria
   * that was used to select information from the framework account
   * table.  It is responsible for building the correct SQL statement,
   * performing the query, and collecting the results.  Furthermore,
   * it is responsible for combining the information from each
   * cassette account extension with the information already gathered
   * for the framework account.
   * <p>
   * A query object trades simplicity for efficiency.  The simplest
   * technique for acquiring cassette dependent information for
   * each account would be to perform a SQL Select statement for
   * each framework account passed to the cassette query object.
   * Instead, this query object performs a single SQL Select statement,
   * joining the cassette dependent account table with the framework
   * account table using the same criteria used to select the framework
   * accounts passed to the cassette query object.
   ********************************************************************/
   public final class AccountQuery extends QueryRequest {

      //<gutter>
      //</gutter>

      /*****************************************************************
      * This query attribute is a collection of all the information
      * selected from the cassette dependent account table.  The key
      * is constructed using the table's primary key columns and the
      * value is an inner account information object.
      *****************************************************************/
      private Hashtable objAccounts = new Hashtable();

      /*****************************************************************
      * This constructor is use by the cassette query object when it
      * is asked to service an AccountQuery request. During construction,
      * it obtains and collects the cassette dependent account
      * information using same criteria that was used to obtain the
      * framework account information.
      *
      * @param     argRequest - the AccountQuery request.
      * @exception ETillAbortOperation - any error results in this
      * exception.
      *****************************************************************/
      public AccountQuery (
         AccountQueryRequest argRequest
      ) throws ETillAbortOperation {
         super(argRequest);
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.AccountQuery()"
            );
         }

         this.query();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.AccountQuery()"
            );
         }
      }

      /*****************************************************************
      * This method is used by this object to select the appropriate
      * columns for the query.
      * <p>
      * For this cassette, all columns are selected.
      *
      * @return - a String that lists the columns to be included in
      * the query's result set.
      *****************************************************************/
      protected String selectStatement() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.selectStatement()"
            );
         }

         String varResult = "*";

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.selectStatement()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to identify the name of
      * the table to be used in the query.
      * <p>
      * For this cassette, the table name is the name of a cassette
      * dependent view that can be found in the database.
      *
      * @return - a String containing the owner qualified table name.
      *****************************************************************/
      protected String tableName() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.tableName()"
            );
         }

         String varResult =
            ETillArchive.geteTillDatabaseOwner() +
            "." +
            VIEW_ACCOUNTS;

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.tableName()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the From
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the method returns the account view selected
      * by this object's tableName() method.
      *
      * @return - The string containing the From clause to be used
      * in the query
      *****************************************************************/
      protected String fromClause() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.fromClause()"
            );
         }

         String varResult = this.tableName();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.fromClause()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this object to construct the Where
      * clause for the SQL Select statement to be performed.
      * <p>
      * For this cassette, the account view selected by this object's
      * tableName() method is designed to use the same Where clause
      * specified in the original request to select the framework
      * account objects.
      *
      * @return - The string containing the Where clause to be used
      * in the query
      *****************************************************************/
      public String whereCondition() {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.whereCondition()"
            );
         }

         QueryRequest varOriginalRequest = this.getOriginalRequest();
         String varResult = varOriginalRequest.whereCondition();

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.whereCondition()"
            );
         }
         return varResult;
      }

      /*****************************************************************
      * This method is used by this class to process the results of
      * the SQL Select statement.  For each row in the result set,
      * it constructs an inner account information object and saves it
      * in a collection using the proper account key.
      *
      * @param     argResultSet - The result set produced when the
      * manufactured SQL Select statement was performed.
      * @return - An integer containing the number of cassette
      * dependent account information objects in the collection.
      * @exception ETillAbortOperation - Any error results in this
      * exception.
      *****************************************************************/
      protected int processQueryResults(
         ResultSet argResultSet
      ) throws ETillAbortOperation {

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.processQueryResults()"
            );
         }

         try {
            while (argResultSet.next()) {
               AccountInfo varAccountInfo = new AccountInfo(argResultSet);
               this.objAccounts.put(
                  varAccountInfo.getKey(),
                  varAccountInfo
               );
            }
         } catch (SQLException varException) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "AccountQuery SQLException caught." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_QUERY_ACC_SELECT_SQL_FAILURE
               );
            }
            ErrorLog.logError(
               RB_ID,
               MSG_QUERY_ACC_SELECT_SQL_FAILURE,
               varException
            );
            throw new ETillAbortOperation(
               PRC_DATABASE_ERROR,
               SRC_QUERY_ACC_SELECT_SQL_FAILURE
            );
         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.processQueryResults()"
            );
         }
         return this.objAccounts.size();
      }

      /*****************************************************************
      * This method is used by the cassette query object to combine
      * cassette dependent account information with framework account
      * information. For each framework account object in the collection
      * provided, it determines if there is a matching cassette account
      * information object and, if so, asks the cassette account
      * information object to add its XML fields to the result.
      *
      * @param     argFrameworkObjects - a collection of framework
      * account objects previously selected by the framework while
      * processing the merchant's query request.
      *****************************************************************/
      public void combine(
         Vector argFrameworkObjects
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionEntry(
               TRACE_ID,
               " AccountQuery.combine()"
            );
         }
         Enumeration varAccountList = argFrameworkObjects.elements();
         while (varAccountList.hasMoreElements()) {
            PSServerAccount varFrameworkAccountInfo =
               (PSServerAccount)varAccountList.nextElement();
            AccountInfo varCassetteAccountInfo =
               (AccountInfo)this.objAccounts.get(
                  new AccountKey(
                     varFrameworkAccountInfo.getMerchantNumber(),
                     varFrameworkAccountInfo.getAccountNumber()
                  )
               );
            if (varCassetteAccountInfo != null) {
               varCassetteAccountInfo.combine(
                  varFrameworkAccountInfo
               );
            }

         }

         if (Trace.isAnyoneTracing()) {
            Trace.traceFunctionExit(
               TRACE_ID,
               " AccountQuery.combine()"
            );
         }
      }

      /*****************************************************************
      * An account information object contains the information gathered
      * from one row of the cassette dependent account table.  It is
      * responsible for providing a hashtable key for the referenced
      * account and for combining its information with matching
      * framework account information.
      *****************************************************************/
      public final class AccountInfo {

         //<gutter>
         //</gutter>

         private String objMerchant        = null;
         private String objAccount         = null;
         private String objCurrency        = null;
         private String objUser            = null;
         private String objPassword        = null;
         private String objLdbCardId       = null;
         private String objUrl             = null;
         private String objBatchCloseTime  = null;
         private String objMinBatchTotal   = null;
         private String objMaxBatchSize    = null;

         /**************************************************************
         * This constructor is used by the account query to save the
         * information selected from a row in the cassette dependent
         * account table.
         *
         * @param     argResultSet - a ResultSet where the current row
         * contains the information to be recorded in this object.
         * @exception SQLException - if an error occurs while reading
         * the columns from the row, this execption is thrown.
         **************************************************************/
         public AccountInfo(
            ResultSet argResultSet
         ) throws SQLException {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " AccountInfo.AccountInfo()"
               );
            }
            this.objMerchant =
               ETillArchive.readString(argResultSet, VIEW_MERCHANT);
            this.objAccount =
               ETillArchive.readString(argResultSet, VIEW_ACCOUNT);
            this.objCurrency =
               ETillArchive.readString(argResultSet, VIEW_CURRENCY);
            this.objUser =
               ETillArchive.readString(argResultSet, VIEW_USER);
            this.objPassword =
               ETillArchive.readString(argResultSet, VIEW_PASSWORD);
            this.objLdbCardId =
               ETillArchive.readString(argResultSet, VIEW_LDBCARD_ID);
            this.objUrl =
               ETillArchive.readString(argResultSet, VIEW_URL);
            this.objBatchCloseTime =
               ETillArchive.readString(argResultSet, VIEW_CLOSE_TIME);
            this.objMinBatchTotal =
               ETillArchive.readString(argResultSet, VIEW_MIN_BATCH_TOTAL);
            this.objMaxBatchSize =
               ETillArchive.readString(argResultSet, VIEW_MAX_BATCH_SIZE);

            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " AccountInfo.AccountInfo()"
               );
            }
         }

         /**************************************************************
         * This method is used by the account query to obtain a key
         * that will uniquely identify this object in a hashtable.
         *
         * @return - a framework account key that uniquely identifies
         * this object.
         **************************************************************/
         public AccountKey getKey() {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " AccountInfo.getKey()"
               );
            }

            AccountKey varAccountKey = new AccountKey (
               this.objMerchant,
               this.objAccount
            );


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " AccountInfo.getKey()"
               );
            }
            return varAccountKey;
         }

         /**************************************************************
         * This method is used by the account query to combine the XML
         * descriptors for this object with the XML descriptors for
         * the matching framework account.
         *
         * @param     argFrameworkAccountInfo - the XML descriptors for
         * the matching framework account.
         **************************************************************/
         public void combine (
            PSServerAccount argFrameworkAccountInfo
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionEntry(
                  TRACE_ID,
                  " AccountInfo.combine()"
               );
            }

            CassetteExtensionObject varAccountXml =
               new CassetteExtensionObject();
            varAccountXml.addProperty(XDM_CURRENCY,this.objCurrency,null);
            varAccountXml.addProperty(XDM_USER,this.objUser,null);
            varAccountXml.addProperty(XDM_PASSWORD,this.objPassword,null);
            varAccountXml.addProperty(XDM_LDBCARD_ID,this.objLdbCardId,null);
            varAccountXml.addProperty(XDM_URL,this.objUrl,null);
            varAccountXml.addProperty(
               XDM_CLOSE_TIME,
               this.objBatchCloseTime,
               null
            );
            varAccountXml.addProperty(
               XDM_MIN_BATCH_TOTAL,
               this.objMinBatchTotal,
               null
            );
            varAccountXml.addProperty(
               XDM_MAX_BATCH_SIZE,
               this.objMaxBatchSize,
               null
            );

            argFrameworkAccountInfo.setCassetteExtensionObject(varAccountXml);


            if (Trace.isAnyoneTracing()) {
               Trace.traceFunctionExit(
                  TRACE_ID,
                  " AccountInfo.combine()"
               );
            }
         }

      }

   }

}
