//**********************************************************************
// This source code is provided as an example.  No warranty is expressed
// or implied.  It is your responsibility to determine its suitability
// to your application, to modify it as necessary, and to validate
// its correctness.
//
// Licensed Materials - Property of IBM
// (c) Copyright IBM Corporation 2000. All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication, or
// disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//**********************************************************************
// Revision / History
// Defect     Date       Who      Description
// ---------- 01/03/2000 ldbader  Module created
//**********************************************************************

package com.ibm.etill.ldbcardcassette;

import  java.sql.Connection;
import  java.sql.PreparedStatement;
import  java.sql.ResultSet;
import  java.sql.SQLException;
import  java.util.Enumeration;
import  java.util.Vector;

import  com.ibm.etill.framework.archive.ETillArchive;
import  com.ibm.etill.framework.cassette.AcceptPaymentRequest;
import  com.ibm.etill.framework.cassette.APIResponse;
import  com.ibm.etill.framework.cassette.ApproveRequest;
import  com.ibm.etill.framework.cassette.ApproveReversalRequest;
import  com.ibm.etill.framework.cassette.Cassette;
import  com.ibm.etill.framework.cassette.CassetteOrder;
import  com.ibm.etill.framework.cassette.CancelOrderRequest;
import  com.ibm.etill.framework.cassette.CloseOrderRequest;
import  com.ibm.etill.framework.cassette.DepositRequest;
import  com.ibm.etill.framework.cassette.DepositReversalRequest;
import  com.ibm.etill.framework.cassette.ReceivePaymentRequest;
import  com.ibm.etill.framework.cassette.RefundRequest;
import  com.ibm.etill.framework.cassette.RefundReversalRequest;
import  com.ibm.etill.framework.clientapi.Base64utils;
import  com.ibm.etill.framework.log.ErrorLog;
import  com.ibm.etill.framework.log.ETillCassetteException;
import  com.ibm.etill.framework.log.Trace;
import  com.ibm.etill.framework.payapi.Amount;
import  com.ibm.etill.framework.payapi.Credit;
import  com.ibm.etill.framework.payapi.ETillAbortOperation;
import  com.ibm.etill.framework.payapi.Order;
import  com.ibm.etill.framework.payapi.ParameterTable;
import  com.ibm.etill.framework.payapi.Payment;
import  com.ibm.etill.framework.payapi.Transaction;
import  com.ibm.etill.framework.supervisor.Supervisor;


/***********************************************************************
* The cassette order object is responsible for recording all the
* relevent protocol data passed when the order is created, managing
* the state of the framework object, and routing all the order
* requests.
***********************************************************************/
public final class LdbCardOrder
   implements CassetteOrder, LdbCardConstants {

   private static final String COPYRIGHT =
      "(c) Copyright IBM Corporation 2000 - All Rights Reserved.";

   //<gutter>
   //</gutter>

   //*******************************************************************
   //This is the cassette order object's instance data.
   //******************************************************************/
   /********************************************************************
   * This cassette order attribute contains the reference back to the
   * cassette that owns the order.
   ********************************************************************/
   private LdbCardCassette       objCassette       = null;
   /********************************************************************
   * This cassette order attribute contains the reference back to the
   * associated framework order object.
   ********************************************************************/
   private Order                 objFrameworkOrder = null;
   /********************************************************************
   * This cassette order attribute contains the reference to the
   * cassette account object that will be used to perform transactions
   * for this order.
   ********************************************************************/
   private LdbCardAccount        objAccount        = null;
   /********************************************************************
   * This cassette order attribute contains the running total of all
   * approved payments associated with this order.  Note that this
   * is a convience attribute since the amount could also be obtained
   * by examining each payment in this order's payment list.
   ********************************************************************/
   private int                   objAmount         = 0;
   /********************************************************************
   * This cassette order attribute contains the shopper's personal
   * account number (also known as the credit/debit card number).
   ********************************************************************/
   private String                objPan            = null;
   /********************************************************************
   * This cassette order attribute contains the expiration date of the
   * shopper's card in YYYYMM format.
   ********************************************************************/
   private String                objExpiry         = null;
   /********************************************************************
   * This cassette order attribute contains the brand name of the
   * shopper's card.
   ********************************************************************/
   private String                objBrand          = null;
   /********************************************************************
   * This cassette order attribute contains the shopper's street
   * address (used for address verification systems).
   ********************************************************************/
   private String                objAddress        = null;
   /********************************************************************
   * This cassette order attribute contains the shopper's postal code
   * (used for address verification systems).
   ********************************************************************/
   private String                objPostalCode     = null;

   /********************************************************************
   * This constructor initializes a new cassette order object using
   * attributes passed by the merchant as protocol data.
   *
   * @param     argCassette - The cassette that owns the order.
   * @param     argFrameworkOrder - The framework's order object.
   * @param     argProtocolData - The protocol data parameter values
   *            provided by the merchant system.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public LdbCardOrder(
      LdbCardCassette argCassette,
      Order           argFrameworkOrder,
      ParameterTable  argProtocolData
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.LdbCardOrder()"
         );
      }

      this.objCassette          = argCassette;
      this.objFrameworkOrder    = argFrameworkOrder;

      // If no protocol data was provided, abort.
      if (argProtocolData == null) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ProtocolData was not provided." +
               "\n\tPRC = " + PRC_PARAMETER_NOT_FOUND +
               "\n\tSRC = " + RC_CASSETTE_PAN
            );
         }
         throw new ETillAbortOperation(
            PRC_PARAMETER_NOT_FOUND,
            RC_CASSETTE_PAN
         );
      }

      // Set the values from the protocol data.
      this.setPan(
         (String)argProtocolData.get(PD_PAY_PAN, null)
      );
      this.setExpiry(
         (String)argProtocolData.get(PD_PAY_EXPIRY, null)
      );
      this.setBrand(
         (String)argProtocolData.get(PD_PAY_BRAND, null)
      );
      this.setAddress(
         (String)argProtocolData.get(PD_PAY_STREETADDRESS, null)
      );
      this.setPostalCode(
         (String)argProtocolData.get(PD_PAY_POSTALCODE, null)
      );

      // Abort if there is no PAN.
      if (this.getPan() == null) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ProtocolData $PAN was not provided." +
               "\n\tPRC = " + PRC_PARAMETER_NOT_FOUND +
               "\n\tSRC = " + RC_CASSETTE_PAN
            );
         }
         throw new ETillAbortOperation(
            PRC_PARAMETER_NOT_FOUND,
            RC_CASSETTE_PAN
         );
      }

      // Abort if there is no Brand.
      if (this.getBrand() == null) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ProtocolData $BRAND was not provided." +
               "\n\tPRC = " + PRC_PARAMETER_NOT_FOUND +
               "\n\tSRC = " + RC_CASSETTE_BRAND
            );
         }
         throw new ETillAbortOperation(
            PRC_PARAMETER_NOT_FOUND,
            RC_CASSETTE_BRAND
         );
      }

      // Abort if there is no Expiry.
      if (this.getExpiry() == null) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ProtocolData $EXPIRY was not provided." +
               "\n\tPRC = " + PRC_PARAMETER_NOT_FOUND +
               "\n\tSRC = " + RC_CASSETTE_EXPIRY
            );
         }
         throw new ETillAbortOperation(
            PRC_PARAMETER_NOT_FOUND,
            RC_CASSETTE_EXPIRY
         );
      }

      // Assign an account to this object.  In this case, by definition,
      // There is only one account per merchant so the account method
      // is used to obtain the proper account object.  Cassettes that
      // allow multiple accounts per merchant must determine which
      // account to use and then pass both the merchant number and
      // the account number to the cassette in order to obtain the
      // proper account object.
      this.objAccount = LdbCardAccount.retrieveAccount(
         this.objFrameworkOrder.getMerchantName()
      );
      // Abort if the account could not be found.
      if (this.objAccount == null) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder No account could be found for merchant." +
               "\n\tPRC = " + PRC_UNDEFINED_OBJECT +
               "\n\tSRC = " + RC_ACCOUNT_ADMIN
            );
         }
         throw new ETillAbortOperation(
            PRC_UNDEFINED_OBJECT,
            RC_ACCOUNT_ADMIN
         );
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.LdbCardOrder()"
         );
      }
   }

   /********************************************************************
   * This constructor initializes an existing cassette order object by
   * resurrecting its attributes from the cassette's order database
   * table.
   *
   * @param     argCassette - The cassette that owns the order.
   * @param     argFrameworkOrder - The framework's order object.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public LdbCardOrder(
      LdbCardCassette argCassette,
      Order           argFrameworkOrder
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.LdbCardOrder()"
         );
      }
      this.objCassette          = argCassette;
      this.objFrameworkOrder    = argFrameworkOrder;
      this.resurrectOrder();

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.LdbCardOrder()"
         );
      }
   }

   /********************************************************************
   * This method is used to obtain the cassette object for the cassette
   * that owns this order.
   *
   * @return - The cassette object for the cassette that owns this
   * order
   ********************************************************************/
   public Cassette cassette() {
      return this.objCassette;
   }

   /********************************************************************
   * This method is used to obtain the account number for the account
   * that will process the order's payments and credits.
   *
   * @return - The account number for the account that will process
   * this order's payments and credits.
   ********************************************************************/
   public String accountNumber() {
      return objAccount.getAccountNumber();
   }

   /********************************************************************
   * The method is used by the framework to determine if a batch ID
   * is required on a Deposit or Refund request.
   * <p>
   * For this cassette, 0 (False) is returned in all cases.  By
   * definition there is only one open batch for each account so there
   * is no need for the merchant to choose a batch with a batch ID.
   *
   * @return 1 (True) if a batch ID is required on Deposit and Refund
   * requests; 0 (False) otherwise.
   ********************************************************************/
   public short batchRequired() {
      return 0;
   }

   /********************************************************************
   * This method is used by the cassette order object to resurrect an
   * existing cassette order from the cassette order database table.
   * <p>
   * This method is <b>not</b> part of the Archivable interface.  The
   * methods for the Archivable interface are passed a JDBC database
   * connection for the operation.  In this case, an ETillArchive
   * object is constructed to provide a JDBC databaes connection.
   *
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   private void resurrectOrder() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.resurrectOrder()"
         );
      }

      // The SQL select statement to obtain the cassette order data.
      String varSelect =
         " SELECT * FROM " +
         ETillArchive.geteTillDatabaseOwner() + "." + TBL_ORDER +
         " WHERE " +
         ORD_MERCHANT + " = '" + this.objFrameworkOrder.getMerchantName() +
         "' AND " +
         ORD_ORDER    + " = '" + this.objFrameworkOrder.getOrderNumber() +
         "'";

      // Construct a JDBC connection.
      ETillArchive varDatabase = new ETillArchive();

      // Lock the JDBC connection.
      synchronized (ETillArchive.getArchiveLock()) {
         try {
            // Trace database read.
            if (Trace.isAnyoneTracing()) {
               Trace.traceDatabaseRead(
                  TRACE_ID,
                  "\n" + varSelect
               );
            }
            // Execute the query.
            ResultSet varResult = varDatabase.beginQuery(varSelect);

            if (varResult.next()) {
               // The required data was found.  Initialize this
               // object's instance data.
               this.setAccount(
                  this.objFrameworkOrder.getMerchantName(),
                  ETillArchive.readString(varResult, ORD_ACCOUNT)
               );
               this.setAmount(
                  ETillArchive.readInteger(varResult, ORD_AMOUNT).intValue()
               );
               this.setPan(
                  ETillArchive.readString(varResult, ORD_PAN)
               );
               this.setBrand(
                  ETillArchive.readString(varResult, ORD_BRAND)
               );
               this.setExpiry(
                  ETillArchive.readString(varResult, ORD_EXPIRY)
               );
               this.setAddress(
                  ETillArchive.readString(varResult, ORD_ADDRESS)
               );
               this.setPostalCode(
                  ETillArchive.readString(varResult, ORD_POSTAL_CODE)
               );
               // Assertion test:  If there is more than one row in the
               // result set, there is a serious defect and the
               // cassette cannot continue.  There should never be more
               // than one row for a given cassette order object.
               // The error is logged and a cassette exception is thrown.
               if (varResult.next()) {
                  if (Trace.isAnyoneTracing()) {
                     Trace.traceErrorOccurred(
                        TRACE_ID,
                        "LdbCardOrder Found too many rows." +
                        "\n\tPRC = " + PRC_DATABASE_ERROR +
                        "\n\tSRC = " + SRC_ORDER_SELECT_ROW_FAILURE
                     );
                  }
                  Vector varVector = new Vector();
                  varVector.addElement(
                     this.objFrameworkOrder.getMerchantName()
                  );
                  varVector.addElement(
                     this.objFrameworkOrder.getOrderNumber()
                  );
                  ErrorLog.log(
                     RB_ID,
                     MSG_ORDER_SELECT_ROW_FAILURE,
                     varVector
                  );
                  throw new ETillCassetteException(
                     ErrorLog.lookupMessage(
                        RB_ID,
                        MSG_ORDER_SELECT_ROW_FAILURE,
                        varVector
                     )
                  );
               }
            } else {
               // The data was not found.  Log an error and abort.
               if (Trace.isAnyoneTracing()) {
                  Trace.traceErrorOccurred(
                     TRACE_ID,
                     "LdbCardOrder Resurrection failed." +
                     "\n\tPRC = " + PRC_DATABASE_ERROR +
                     "\n\tSRC = " + SRC_ORDER_SELECT_ORDER_MISSING
                  );
               }
               ErrorLog.logError(
                  RB_ID,
                  MSG_ORDER_SELECT_ORDER_MISSING,
                  null,
                  this.objFrameworkOrder.getMerchantName(),
                  this.objFrameworkOrder.getOrderNumber()
               );
               throw new ETillAbortOperation(
                  PRC_DATABASE_ERROR,
                  SRC_ORDER_SELECT_ORDER_MISSING
               );
            }
         } catch (SQLException varException) {
            // The query failed.  Log the error and abort.
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder SQLException caught." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_ORDER_SELECT_SQL_FAILURE
               );
            }
            ErrorLog.logError(
               RB_ID,
               MSG_ORDER_SELECT_SQL_FAILURE,
               varException,
               this.objFrameworkOrder.getMerchantName(),
               this.objFrameworkOrder.getOrderNumber()
            );
            throw new ETillAbortOperation(
               PRC_DATABASE_ERROR,
               SRC_ORDER_SELECT_SQL_FAILURE
            );
         } finally {
            // Regardless of whether there was an exception or not,
            // clean up the ETillArchive object.
            try {
               varDatabase.endQuery();
            } catch (SQLException varException) {
               // If there was an error, it occured during cleanup so
               // there is no reason to abort.  Just log the error
               // because there is a defect in the ETillArchive object.
               if (Trace.isAnyoneTracing()) {
                  Trace.traceErrorOccurred(
                     TRACE_ID,
                     "LdbCardOrder SQL Exception caught." +
                     "\n\tPRC = " + PRC_DATABASE_ERROR +
                     "\n\tSRC = " + SRC_ORDER_SELECT_CLOSE_FAILURE
                  );
               }
               ErrorLog.logError(
                  RB_ID,
                  MSG_ORDER_SELECT_CLOSE_FAILURE,
                  varException,
                  this.objFrameworkOrder.getMerchantName(),
                  this.objFrameworkOrder.getOrderNumber()
               );
            }
         }
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.resurrectOrder()"
         );
      }
   }

   /********************************************************************
   * This method is used by the framework order (a commitable object)
   * when its commit() method discovers the cassette order object on
   * the framework order's create list.
   *
   * @param     argConnection - The JDBC database connection to be used
   * for this operation.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void createRecord(
      Connection argConnection
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.createRecord()"
         );
      }
      try {
         // The SQL statement to insert a new cassette order.
         String varInsert =
            " INSERT INTO " +
            ETillArchive.geteTillDatabaseOwner() + "." + TBL_ORDER + " (" +
            ORD_MERCHANT     + ", " +
            ORD_ORDER        + ", " +
            ORD_ACCOUNT      + ", " +
            ORD_AMOUNT       + ", " +
            ORD_PAN          + ", " +
            ORD_BRAND        + ", " +
            ORD_EXPIRY       + ", " +
            ORD_ADDRESS      + ", " +
            ORD_POSTAL_CODE  +
            ") VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)";
         if (Trace.isAnyoneTracing()) {
            Trace.traceDatabaseWrite(
               TRACE_ID,
               "\n" + varInsert + "\n" +
               ORD_MERCHANT   + " = " +
               this.objFrameworkOrder.getMerchantName() + "\n" +
               ORD_ORDER      + " = " +
               this.objFrameworkOrder.getOrderNumber() + "\n" +
               ORD_ACCOUNT    + " = " + this.getAccountNumber() + "\n" +
               ORD_AMOUNT     + " = " + this.getAmount() + "\n" +
               ORD_PAN        + " = " + this.getObscurePan() + "\n" +
               ORD_BRAND      + " = " + this.getBrand() + "\n" +
               ORD_EXPIRY     + " = " + this.getExpiry() + "\n" +
               ORD_ADDRESS    + " = " + this.getAddress() + "\n" +
               ORD_POSTAL_CODE+ " = " + this.getPostalCode() + "\n"
            );
         }
         // Ask the connection to build a prepared statement using
         // the SQL statement defined above.
         PreparedStatement varStatement =
            argConnection.prepareStatement(varInsert);
         // Clear all the variable parameters from the statement.
         varStatement.clearParameters();
         // Tell the statement to set the variable values.
         ETillArchive.addString(
            this.objFrameworkOrder.getMerchantName(),    varStatement, 1
         );
         ETillArchive.addString(
            this.objFrameworkOrder.getOrderNumber(),     varStatement, 2
         );
         ETillArchive.addString(
            this.getAccountNumber(),                     varStatement, 3
         );
         ETillArchive.addInteger(
            new Integer(this.getAmount()),               varStatement, 4
         );
         ETillArchive.addString(
            this.getPan(),                               varStatement, 5
         );
         ETillArchive.addString(
            this.getBrand(),                             varStatement, 6
         );
         ETillArchive.addString(
            this.getExpiry(),                            varStatement, 7
         );
         ETillArchive.addString(
            this.getAddress(),                           varStatement, 8
         );
         ETillArchive.addString(
            this.getPostalCode(),                        varStatement, 9
         );
         // Execute the insert.
         int varRowsAffected = varStatement.executeUpdate();
         // Close the statement.
         varStatement.close();
         // Assertion test:  If the number of affected rows is not 1
         // there is a serious defect and the cassette cannot continue.
         // The error is logged and a cassette exception is thrown.
         if (varRowsAffected != 1) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder Created too many rows." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_ORDER_CREATE_ROW_FAILURE
               );
            }
            Vector varVector = new Vector();
            varVector.addElement(
               this.objFrameworkOrder.getMerchantName()
            );
            varVector.addElement(
               this.objFrameworkOrder.getOrderNumber()
            );
            varVector.addElement(Integer.toString(varRowsAffected));
            ErrorLog.log(
               RB_ID,
               MSG_ORDER_CREATE_ROW_FAILURE,
               varVector
            );
            throw new ETillCassetteException(
               ErrorLog.lookupMessage(
                  RB_ID,
                  MSG_ORDER_CREATE_ROW_FAILURE,
                  varVector
               )
            );
         }
      }  catch (SQLException varException) {
         // The insert failed.  Log the error and abort.
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder SQLException caught." +
               "\n\tPRC = " + PRC_DATABASE_ERROR +
               "\n\tSRC = " + SRC_ORDER_CREATE_SQL_FAILURE
            );
         }
         ErrorLog.logError(
            RB_ID,
            MSG_ORDER_CREATE_SQL_FAILURE,
            varException,
            this.objFrameworkOrder.getMerchantName(),
            this.objFrameworkOrder.getOrderNumber()
         );
         throw new ETillAbortOperation(
            PRC_DATABASE_ERROR,
            SRC_ORDER_CREATE_SQL_FAILURE
         );
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.createRecord()"
         );
      }
   }

   /********************************************************************
   * This method is used by the framework order (a commitable object)
   * when its commit() method discovers the cassette order object on
   * the framework order's update list.
   *
   * @param     argConnection - The JDBC database connection to be used
   * for this operation.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void updateRecord(
      Connection argConnection
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.updateRecord()"
         );
      }
      try {
         // The SQL statement to update an existing cassette order.
         String varUpdate =
            " UPDATE " +
            ETillArchive.geteTillDatabaseOwner() + "." + TBL_ORDER + " SET " +
            ORD_ACCOUNT      + "= ?, " +
            ORD_AMOUNT       + "= ?, " +
            ORD_PAN          + "= ?, " +
            ORD_BRAND        + "= ?, " +
            ORD_EXPIRY       + "= ?, " +
            ORD_ADDRESS      + "= ?, " +
            ORD_POSTAL_CODE  + "= ? " +
            " WHERE " +
            ORD_MERCHANT + " = ? AND " +
            ORD_ORDER    + " = ?";
         if (Trace.isAnyoneTracing()) {
            Trace.traceDatabaseWrite(
               TRACE_ID,
               "\n" + varUpdate + "\n" +
               ORD_ACCOUNT    + " = " + this.getAccountNumber() + "\n" +
               ORD_AMOUNT     + " = " + this.getAmount() + "\n" +
               ORD_PAN        + " = " + this.getObscurePan() + "\n" +
               ORD_BRAND      + " = " + this.getBrand() + "\n" +
               ORD_EXPIRY     + " = " + this.getExpiry() + "\n" +
               ORD_ADDRESS    + " = " + this.getAddress() + "\n" +
               ORD_POSTAL_CODE+ " = " + this.getPostalCode() + "\n" +
               ORD_MERCHANT   + " = " +
               this.objFrameworkOrder.getMerchantName() + "\n" +
               ORD_ORDER      + " = " +
               this.objFrameworkOrder.getOrderNumber() + "\n"
            );
         }
         // Ask the connection to build a prepared statement using
         // the SQL statement defined above.
         PreparedStatement varStatement =
            argConnection.prepareStatement(varUpdate);
         // Clear all the variable parameters from the statement.
         varStatement.clearParameters();
         // Tell the statement to set the variable values.
         ETillArchive.addString(
            this.getAccountNumber(),                     varStatement, 1
         );
         ETillArchive.addInteger(
            new Integer(this.getAmount()),               varStatement, 2
         );
         ETillArchive.addString(
            this.getPan(),                               varStatement, 3
         );
         ETillArchive.addString(
            this.getBrand(),                             varStatement, 4
         );
         ETillArchive.addString(
            this.getExpiry(),                            varStatement, 5
         );
         ETillArchive.addString(
            this.getAddress(),                           varStatement, 6
         );
         ETillArchive.addString(
            this.getPostalCode(),                        varStatement, 7
         );
         ETillArchive.addString(
            this.objFrameworkOrder.getMerchantName(),    varStatement, 8
         );
         ETillArchive.addString(
            this.objFrameworkOrder.getOrderNumber(),     varStatement, 9
         );
         // Execute the update.
         int varRowsAffected = varStatement.executeUpdate();
         // Close the statement.
         varStatement.close();
         // Assertion test:  If the number of affected rows is not 1
         // there is a serious defect and the cassette cannot continue.
         // The error is logged and a cassette exception is thrown.
         if (varRowsAffected != 1) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder Modified too many rows." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_ORDER_UPDATE_ROW_FAILURE
               );
            }
            Vector varVector = new Vector();
            varVector.addElement(
               this.objFrameworkOrder.getMerchantName()
            );
            varVector.addElement(
               this.objFrameworkOrder.getOrderNumber()
            );
            varVector.addElement(Integer.toString(varRowsAffected));
            ErrorLog.log(
               RB_ID,
               MSG_ORDER_UPDATE_ROW_FAILURE,
               varVector
            );
            throw new ETillCassetteException(
               ErrorLog.lookupMessage(
                  RB_ID,
                  MSG_ORDER_UPDATE_ROW_FAILURE,
                  varVector
               )
            );
         }
      }  catch (SQLException varException) {
         // The update failed.  Log the error and abort.
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder SQLException caught." +
               "\n\tPRC = " + PRC_DATABASE_ERROR +
               "\n\tSRC = " + SRC_ORDER_UPDATE_SQL_FAILURE
            );
         }
         ErrorLog.logError(
            RB_ID,
            MSG_ORDER_UPDATE_SQL_FAILURE,
            varException,
            this.objFrameworkOrder.getMerchantName(),
            this.objFrameworkOrder.getOrderNumber()
         );
         throw new ETillAbortOperation(
            PRC_DATABASE_ERROR,
            SRC_ORDER_UPDATE_SQL_FAILURE
         );
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.updateRecord()"
         );
      }
   }

   /********************************************************************
   * This method is used by the framework order (a commitable object)
   * when its commit() method discovers the cassette order object on
   * the framework order's delete list.
   *
   * @param     argConnection - The JDBC database connection to be used
   * for this operation.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void deleteRecord(
      Connection argConnection
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.deleteRecord()"
         );
      }
      try {
         // The SQL statement to delete an existing cassette order.
         String varDelete =
            " DELETE FROM " +
            ETillArchive.geteTillDatabaseOwner() + "." + TBL_ORDER +
            " WHERE " +
            ORD_MERCHANT + " = ? AND " +
            ORD_ORDER    + " = ?";
         if (Trace.isAnyoneTracing()) {
            Trace.traceDatabaseWrite(
               TRACE_ID,
               "\n" + varDelete + "\n" +
               ORD_MERCHANT   + " = " +
               this.objFrameworkOrder.getMerchantName() + "\n" +
               ORD_ORDER      + " = " +
               this.objFrameworkOrder.getOrderNumber() + "\n"
            );
         }
         // Ask the connection to build a prepared statement using
         // the SQL statement defined above.
         PreparedStatement varStatement =
            argConnection.prepareStatement(varDelete);
         // Clear all the variable parameters from the statement.
         varStatement.clearParameters();
         // Tell the statement to set the variable values.
         ETillArchive.addString(
            this.objFrameworkOrder.getMerchantName(),    varStatement, 1
         );
         ETillArchive.addString(
            this.objFrameworkOrder.getOrderNumber(),     varStatement, 2
         );
         // Execute the delete.
         int varRowsAffected = varStatement.executeUpdate();
         // Close the statement.
         varStatement.close();
         // Assertion test:  If the number of affected rows is not 1
         // there is a serious defect and the cassette cannot continue.
         // The error is logged and a cassette exception is thrown.
         if (varRowsAffected != 1) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder Deleted too many rows." +
                  "\n\tPRC = " + PRC_DATABASE_ERROR +
                  "\n\tSRC = " + SRC_ORDER_DELETE_SQL_FAILURE
               );
            }
            Vector varVector = new Vector();
            varVector.addElement(
               this.objFrameworkOrder.getMerchantName()
            );
            varVector.addElement(
               this.objFrameworkOrder.getOrderNumber()
            );
            varVector.addElement(Integer.toString(varRowsAffected));
            ErrorLog.log(
               RB_ID,
               MSG_ORDER_DELETE_ROW_FAILURE,
               varVector
            );
            throw new ETillCassetteException(
               ErrorLog.lookupMessage(
                  RB_ID,
                  MSG_ORDER_DELETE_ROW_FAILURE,
                  varVector
               )
            );
         }
      }  catch (SQLException varException) {
         // The delete failed.  Log the error and abort.
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder SQLException caught." +
               "\n\tPRC = " + PRC_DATABASE_ERROR +
               "\n\tSRC = " + SRC_ORDER_DELETE_SQL_FAILURE
            );
         }
         ErrorLog.logError(
            RB_ID,
            MSG_ORDER_DELETE_SQL_FAILURE,
            varException,
            this.objFrameworkOrder.getMerchantName(),
            this.objFrameworkOrder.getOrderNumber()
         );
         throw new ETillAbortOperation(
            PRC_DATABASE_ERROR,
            SRC_ORDER_DELETE_SQL_FAILURE
         );
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.deleteRecord()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a ReceivePayment request.
   * <p>
   * For this cassette, the payment engine does not support shopper
   * interaction to obtain payment information after the order is
   * created.  The operation is always aborted.
   *
   * @param     argRequest - A ReceivePayment order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void receivePayment(
      ReceivePaymentRequest  argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.receivePayment()"
         );
      }
      // Abort ReceivePayment if the Order is not in RESET state.
      if (this.objFrameworkOrder.getCurrentState() != ORDER_RESET) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ReceivePayment was requested but the" +
               " order is not in ORDER_RESET state."+
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Abort ReceivePayment because it is not supported by this
      // cassette.
      if (Trace.isAnyoneTracing()) {
         Trace.traceErrorOccurred(
            TRACE_ID,
            "LdbCardOrder ReceivePayment is not supported." +
            "\n\tPRC = " + PRC_COMMAND_NOT_SUPPORTED +
            "\n\tSRC = " + RC_NONE
         );
      }
      throw new ETillAbortOperation(
         PRC_COMMAND_NOT_SUPPORTED,
         RC_NONE
      );

      // Remove comments when ReceivePayment is supported.
      //if (Trace.isAnyoneTracing()) {
      //   Trace.traceFunctionExit(
      //      TRACE_ID,
      //      "LdbCardOrder.receivePayment()"
      //   );
      //}
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * an AcceptPayment request.
   * <p>
   * For this cassette, required and optional protocol data values have
   * already been saved by the constructor.  This method peforms the
   * state checks, assignes an account to process subsequent financial
   * requests, updates the framework order's state, and adds the
   * framework order to the current commit point. In an automatic
   * approval has been requested, it is performed.
   *
   * @param     argRequest - An AcceptPayment order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void acceptPayment(
      AcceptPaymentRequest   argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.acceptPayment()"
         );
      }
      // Abort AcceptPayment if the Order is not in RESET state.
      if (this.objFrameworkOrder.getCurrentState() != ORDER_RESET) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder AcceptPayment was requested but the" +
               " order is not in ORDER_RESET state."+
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }

      // Abort if the currency in the account is different than
      // the currency in the order.
      if (
         !this.objFrameworkOrder.getAmount().getCurrencyAsString().
         equals(this.objAccount.getCurrency())
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder No account and order currency mismatch." +
               "\n\tPRC = " + PRC_PARAMETER_MISMATCH +
               "\n\tSRC = " + RC_CURRENCY
            );
         }
         throw new ETillAbortOperation(
            PRC_PARAMETER_MISMATCH,
            RC_CURRENCY
         );
      }

      // Set the state in the framework order.  In this case,
      // the state is set to Refundable because the payment
      // engine allows credits even when there are no payments.
      this.objFrameworkOrder.setCurrentState(ORDER_REFUNDABLE);
      // Add the framework order to the create list.  There is no
      // need to add the cassette order object to the create list
      // because the framework order will automatically call the
      // cassette order's createRecord() method from within its own
      // createRecord() method.
      Supervisor.getThreadCommitPoint().
         addToCreateList(this.objFrameworkOrder);
      // Set the return codes to indicate a successful operation.
      argResponse.setReturnCodes(PRC_OPERATION_SUCCESS, RC_NONE);

      // Check if the merchant requested an automatic approval.
      if (argRequest.getApproveFlag()) {
         // An automatic approval has been requested.
         // Construct a new Amount object using the currency and
         // exponent from the order's amount and an amount that
         // was specified as the initial payment.
         Amount varAmount = new Amount(
            this.objFrameworkOrder.getAmount().getCurrency(),
            this.objFrameworkOrder.getInitialPaymentAmount(),
            this.objFrameworkOrder.getAmount().getAmountExp10()
         );

         // Ask the Supervisor to create a new framework payment.
         // The cassette's newCassettePayment() method will be called
         // automatically during this procedure.
         Payment varFrameworkPayment = Supervisor.createPayment(
            this.objFrameworkOrder,
            this.objFrameworkOrder.getInitialPaymentNumber(),
            varAmount,
            argRequest.getProtocolData()
         );

         // Construct an ApproveReuest and initialize its values.
         ApproveRequest varApproveRequest = new ApproveRequest(
            argRequest.getConnection(),
            varFrameworkPayment,
            varAmount,
            true,
            argRequest.getDepositFlag(),
            argRequest.getBatchNumber(),
            argRequest.getFrameworkKeywords(),
            argRequest.getProtocolData()
         );

         // Service the new approve request.
         this.approve(varApproveRequest, argResponse);

      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.acceptPayment()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * an Approve request.
   * <p>
   * For this cassette, the cassette order object knows that the cassette
   * payment object is responsible for performing the approval.  The
   * framework's payment object is found in the request.  The cassette's
   * payment object is obtained and its approve() method is called.
   *
   * @param     argRequest - An Approve order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void approve(
      ApproveRequest         argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.approve()"
         );
      }
      // Abort Approve if order is not in ORDERED or REFUNDABLE state.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED  &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder Approve was requested but the" +
               " order is not in ORDER_ORDERED or ORDER_REFUNDABLE state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Obtain the transaction from the request.  A Transaction is
      // a superclass for both credit and payment objects, but in this
      // case, we know it must contain a framework payment object.
      Payment varFrameworkPayment = (Payment)argRequest.getTransaction();

      // Obtain the cassette's payment object from the framework
      // payment object and call its approve() method.
      LdbCardPayment varCassettePayment =
         (LdbCardPayment)varFrameworkPayment.getCassetteTransaction();
      varCassettePayment.approve(
         argRequest,
         argResponse
      );

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.approve()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * an ApproveReversal request.
   * <p>
   * For this cassette, the cassette order object knows that the cassette
   * payment object is responsible for performing the reversal.  The
   * framework's payment object is found in the request.  The cassette's
   * payment object is obtained and its approveReversal() method is
   * called.
   *
   * @param     argRequest - An ApproveReversal order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void approveReversal(
      ApproveReversalRequest argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.approveReversal()"
         );
      }
      // Abort ApproveReversal if order is not in ORDERED or REFUNDABLE state.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED    &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ApproveReversal was requested but the" +
               " order is not in ORDER_ORDERED or ORDER_REFUNDABLE state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Obtain the transaction from the request.  A Transaction is
      // a superclass for both credit and payment objects, but in this
      // case, we know it must contain a framework payment object.
      Payment varFrameworkPayment =
         (Payment)argRequest.getTransaction();
      // Obtain the cassette's payment object from the framework
      // payment object and call its approveReversal() method.
      LdbCardPayment varCassettePayment =
         (LdbCardPayment)varFrameworkPayment.getCassetteTransaction();
      varCassettePayment.approveReversal(
         argRequest,
         argResponse
      );

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.approveReversal()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a Deposit request.
   * <p>
   * For this cassette, the cassette order object knows that the cassette
   * payment object is responsible for performing the deposit.  The
   * framework's payment object is found in the request.  The cassette's
   * payment object is obtained and its deposit() method is called.
   *
   * @param     argRequest - A Deposit order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void deposit(
      DepositRequest         argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.deposit()"
         );
      }
      // Abort Deposit if order is not in ORDERED or REFUNDABLE state.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED  &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder Deposit was requested but the" +
               " order is not in ORDER_ORDERED or ORDER_REFUNDABLE state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Obtain the transaction from the request.  A Transaction is
      // a superclass for both credit and payment objects, but in this
      // case, we know it must contain a framework payment object.
      Payment varFrameworkPayment =
         (Payment)argRequest.getTransaction();
      // Obtain the cassette's payment object from the framework
      // payment object and call its deposit() method.
      LdbCardPayment varCassettePayment =
         (LdbCardPayment)varFrameworkPayment.getCassetteTransaction();
      varCassettePayment.deposit(
         argRequest,
         argResponse
      );

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.deposit()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a DepositReversal request.
   * <p>
   * For this cassette, the cassette order object knows that the cassette
   * payment object is responsible for performing the reversal.  The
   * framework's payment object is found in the request.  The cassette's
   * payment object is obtained and its depositReversal() method is
   * called.
   *
   * @param     argRequest - A DepositReversal order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void depositReversal(
      DepositReversalRequest argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.depositReversal()"
         );
      }
      // Abort DepositReversal if order is not in ORDERED or REFUNDABLE state.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED  &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder DepositReversal was requested but the" +
               " order is not in ORDER_ORDERED or ORDER_REFUNDABLE state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Obtain the transaction from the request.  A Transaction is
      // a superclass for both credit and payment objects, but in this
      // case, we know it must contain a framework payment object.
      Payment varFrameworkPayment =
         (Payment)argRequest.getTransaction();
      // Obtain the cassette's payment object from the framework
      // payment object and call its depositReversal() method.
      LdbCardPayment varCassettePayment =
         (LdbCardPayment)varFrameworkPayment.getCassetteTransaction();
      varCassettePayment.depositReversal(
         argRequest,
         argResponse
      );

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.depositReversal()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a Refund request.
   * <p>
   * For this cassette, the cassette order object knows that the cassette
   * credit object is responsible for performing the refund.  The
   * framework's credit object is found in the request.  The cassette's
   * credit object is obtained and its refund() method is called.
   *
   * @param     argRequest - A Refund order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void refund(
      RefundRequest          argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.refund()"
         );
      }
      // Abort Refund if order is not in REFUNDABLE state.
      if (this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder Refund was requested but the" +
               " order is not in ORDER_REFUNDABLE state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Obtain the transaction from the request.  A Transaction is
      // a superclass for both credit and payment objects, but in this
      // case, we know it must contain a framework credit object.
      Credit varFrameworkCredit = (Credit)argRequest.getTransaction();

      // Obtain the cassette's credit object from the framework
      // credit object and call its refund() method.
      LdbCardCredit varCassetteCredit =
         (LdbCardCredit)varFrameworkCredit.getCassetteTransaction();
      varCassetteCredit.refund(
         argRequest,
         argResponse
      );

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.refund()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a RefundReversal request.
   * <p>
   * For this cassette, the cassette order object knows that the cassette
   * credit object is responsible for performing the reversal.  The
   * framework's credit object is found in the request.  The cassette's
   * credit object is obtained and its refundReversal() method is
   * called.
   *
   * @param     argRequest - A Refund order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void refundReversal(
      RefundReversalRequest  argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.refundReversal()"
         );
      }
      // Abort RefundReversal if order is not in REFUNDABLE state.
      if ( this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder RefundReversal was requested but the" +
               " order is not in ORDER_REFUNDABLE state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      // Obtain the transaction from the request.  A Transaction is
      // a superclass for both credit and payment objects, but in this
      // case, we know it must contain a framework credit object.
      Credit varFrameworkCredit = (Credit)argRequest.getTransaction();
      // Obtain the cassette's credit object from the framework
      // credit object and call its refundReversal() method.
      LdbCardCredit varCassetteCredit =
         (LdbCardCredit)varFrameworkCredit.getCassetteTransaction();
      varCassetteCredit.refundReversal(
         argRequest,
         argResponse
      );

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.refundReversal()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a CloseOrder request.
   * <p>
   *
   * @param     argRequest - A CloseOrder order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void closeOrder(
      CloseOrderRequest      argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.closeOrder()"
         );
      }

      // Abort CloseOrder if order is not in ORDERED, REFUNDABLE, or
      // CLOSED state.  Also, abort if there are active payments or
      // credits linked to this order.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED  &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE &&
         this.objFrameworkOrder.getCurrentState() != ORDER_CLOSED
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder CloseOrder was requested but the" +
               " order is not in ORDER_ORDERED, ORDER_REFUNDABLE, or" +
               " ORDER_CLOSED state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      if (!this.isCloseOrderAllowed()) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder CloseOrder was requested but the" +
               " order contains active payments and/or credits." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }

      // Everything is fine, so set the order's state to closed and
      // either prune the order (if a delete is requested) or add the
      // order to the update list.
      this.objFrameworkOrder.setCurrentState(ORDER_CLOSED);
      if (argRequest.getDeleteOrder()) {
         Supervisor.pruneOrder(objFrameworkOrder);
      } else {
         Supervisor.getThreadCommitPoint().addToUpdateList(
            this.objFrameworkOrder
         );
      }
      // Set the return codes to indicate a successful operation.
      argResponse.setReturnCodes(PRC_OPERATION_SUCCESS, RC_NONE);

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.closeOrder()"
         );
      }
   }

   /********************************************************************
   * This method is used to determine if a CloseOrder command is
   * allowed.  By definition, a CloseOrder is allowed if there are no
   * active payments or credits.
   * <p>
   * Note:  At the time of this implementation, this definition is
   * contrary to the definition in the programmer's guide.  The
   * programmer's guide contains a much stricter definition: "You
   * can invoke the CLOSEORDER command when there is at least one
   * payment or credit in Closed state [and there are no active
   * payments or credits]."  I cannot justify this definition.  If
   * this definition is used, a merchant who is pruning their orders
   * would need to issue a CloseOrder for every order that contains
   * a closed payment or credit and a CancelOrder for everything else.
   * Imagine the complicated code required to choose which verb should
   * be issued for hundreds of orders.  When compared to this
   * complication, the feeble attempt to prevent a merchant from closing
   * an order too soon by requiring a closed payment or credit seems
   * pointless.  The active transaction check is both necessary and
   * sufficient.  In the near future, the definition will be changed.
   *
   * @return    True if a CloseOrder command is allowed, false
   * otherwise.
   ********************************************************************/
   private boolean isCloseOrderAllowed() {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.closeOrderAllowed()"
         );
      }

      boolean varResult = false;
      int varActivePayments = 0;
      int varActiveCredits = 0;

      // When closing an order, a payment is considered active
      // unless its current state is PAYMENT_VOID, PAYMENT_DECLINED,
      // PAYMENT_RESET, or PAYMENT_CLOSED.
      Enumeration varFrameworkPaymentList =
         this.objFrameworkOrder.allPayments();
      while (varFrameworkPaymentList.hasMoreElements()) {
         Payment varFrameworkPayment =
            (Payment)varFrameworkPaymentList.nextElement();
         if (
            varFrameworkPayment.getCurrentState() != PAYMENT_VOID     &&
            varFrameworkPayment.getCurrentState() != PAYMENT_DECLINED &&
            varFrameworkPayment.getCurrentState() != PAYMENT_CLOSED   &&
            varFrameworkPayment.getCurrentState() != PAYMENT_RESET
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder CloseOrder was requested but the" +
                  " order contains an active payment." +
                  "\n\tMerchant = " + varFrameworkPayment.getMerchantNumber() +
                  "\n\tOrder    = " + varFrameworkPayment.getOrderNumber() +
                  "\n\tPayment  = " + varFrameworkPayment.getTransactionNumber() +
                  "\n\tState    = " + varFrameworkPayment.getCurrentState()
               );
            }
            varActivePayments++;
         }
      }

      // When closing an order, a credit is considered active
      // unless its current state is CREDIT_VOID, CREDIT_DECLINED,
      // CREDIT_RESET, or CREDIT_CLOSED.
      Enumeration varFrameworkCreditList =
         this.objFrameworkOrder.allCredits();
      while (varFrameworkCreditList.hasMoreElements()) {
         Credit varFrameworkCredit =
            (Credit)varFrameworkCreditList.nextElement();
         if (
            varFrameworkCredit.getCurrentState() != CREDIT_VOID     &&
            varFrameworkCredit.getCurrentState() != CREDIT_DECLINED &&
            varFrameworkCredit.getCurrentState() != CREDIT_CLOSED   &&
            varFrameworkCredit.getCurrentState() != CREDIT_RESET
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder CloseOrder was requested but the" +
                  " order contains an active credit." +
                  "\n\tMerchant = " + varFrameworkCredit.getMerchantNumber() +
                  "\n\tOrder    = " + varFrameworkCredit.getOrderNumber() +
                  "\n\tCredit   = " + varFrameworkCredit.getTransactionNumber() +
                  "\n\tState    = " + varFrameworkCredit.getCurrentState()
               );
            }
            varActiveCredits++;
         }
      }

      if ((varActivePayments + varActiveCredits) == 0) {
         varResult = true;
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.closeOrderAllowed()"
         );
      }
      return varResult;
   }

   /********************************************************************
   * This method is used by the cassette when it is asked to service
   * a CancelOrder request.
   * <p>
   *
   * @param     argRequest - A CancelOrder order request.
   * @param     argResponse - The response object to hold the result.
   * @exception ETillAbortOperation - any failure results in this
   * exception.
   ********************************************************************/
   public void cancelOrder(
      CancelOrderRequest     argRequest,
      APIResponse            argResponse
   ) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.cancelOrder()"
         );
      }

      // Abort CancelOrder if order is not in ORDERED, REFUNDABLE, or
      // CLOSED state.  Also, abort if there are any active payments
      // or credits linked to this order.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED  &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE &&
         this.objFrameworkOrder.getCurrentState() != ORDER_CANCELED
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder CancelOrder was requested but the" +
               " order is not in ORDER_ORDERED, ORDER_REFUNDABLE, or" +
               " ORDER_CANCELED state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }
      if (!this.isCancelOrderAllowed()) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder CancelOrder was requested but the" +
               " order contains active payments and/or credits." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         throw new ETillAbortOperation (
            PRC_VERB_NOT_VALID_IN_PRESENT_STATE,
            RC_ORDER
         );
      }

      // Everything is fine, so set the order's state to canceled and
      // either prune the order (if a delete is requested) or add the
      // order to the update list.
      this.objFrameworkOrder.setCurrentState(ORDER_CANCELED);
      if (argRequest.getDeleteOrder()) {
         Supervisor.pruneOrder(objFrameworkOrder);
      } else {
         Supervisor.getThreadCommitPoint().addToUpdateList(
            this.objFrameworkOrder
         );
      }
      // Set the return codes to indicate a successful operation.
      argResponse.setReturnCodes(PRC_OPERATION_SUCCESS, RC_NONE);

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.cancelOrder()"
         );
      }
   }


   /********************************************************************
   * This method is used to determine if a CancelOrder command is
   * allowed.  By definition, a CancelOrder is allowed if there are
   * no active payements or credits.
   *
   * @return    True if a CancelOrder command is allowed, false
   * otherwise.
   ********************************************************************/
   private boolean isCancelOrderAllowed() {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.cancelOrderAllowed()"
         );
      }

      boolean varResult = false;
      int varActivePayments = 0;
      int varActiveCredits = 0;

      // When canceling an order, a payment is considered active
      // unless its current state is PAYMENT_VOID, PAYMENT_DECLINED, or
      // PAYMENT_RESET.
      Enumeration varFrameworkPaymentList =
         this.objFrameworkOrder.allPayments();
      while (varFrameworkPaymentList.hasMoreElements()) {
         Payment varFrameworkPayment =
            (Payment)varFrameworkPaymentList.nextElement();
         if (
            varFrameworkPayment.getCurrentState() != PAYMENT_VOID     &&
            varFrameworkPayment.getCurrentState() != PAYMENT_DECLINED &&
            varFrameworkPayment.getCurrentState() != PAYMENT_RESET
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder CancelOrder was requested but the" +
                  " order contains an active payment." +
                  "\n\tMerchant = " + varFrameworkPayment.getMerchantNumber() +
                  "\n\tOrder    = " + varFrameworkPayment.getOrderNumber() +
                  "\n\tPayment  = " + varFrameworkPayment.getTransactionNumber() +
                  "\n\tState    = " + varFrameworkPayment.getCurrentState()
               );
            }
            varActivePayments++;
         }
      }

      // When canceling an order, a credit is considered active
      // unless its current state is CREDIT_VOID, CREDIT_DECLINED, or
      // CREDIT_RESET.
      Enumeration varFrameworkCreditList =
         this.objFrameworkOrder.allCredits();
      while (varFrameworkCreditList.hasMoreElements()) {
         Credit varFrameworkCredit =
            (Credit)varFrameworkCreditList.nextElement();
         if (
            varFrameworkCredit.getCurrentState() != CREDIT_VOID     &&
            varFrameworkCredit.getCurrentState() != CREDIT_DECLINED &&
            varFrameworkCredit.getCurrentState() != CREDIT_RESET
         ) {
            if (Trace.isAnyoneTracing()) {
               Trace.traceErrorOccurred(
                  TRACE_ID,
                  "LdbCardOrder CancelOrder was requested but the" +
                  " order contains an active credit." +
                  "\n\tMerchant = " + varFrameworkCredit.getMerchantNumber() +
                  "\n\tOrder    = " + varFrameworkCredit.getOrderNumber() +
                  "\n\tCredit   = " + varFrameworkCredit.getTransactionNumber() +
                  "\n\tState    = " + varFrameworkCredit.getCurrentState()
               );
            }
            varActiveCredits++;
         }
      }

      if ((varActivePayments + varActiveCredits) == 0) {
         varResult = true;
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.cancelOrderAllowed()"
         );
      }
      return varResult;
   }

   /********************************************************************
   * This method is used by the cassette batch when it is asked to close
   * an open batch.  It is called for every payment found in the open
   * batch.
   * <p>
   *
   * @param     argFrameworkPayment - A framework payment to be closed.
   ********************************************************************/
   public void closePayment(
      Payment                argFrameworkPayment
   ) {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.closePayment()"
         );
      }

      // Assertion test.  If the order is not in the correct state,
      // there is a serious defect and the cassette cannot continue.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_ORDERED  &&
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE &&
         this.objFrameworkOrder.getCurrentState() != ORDER_CLOSED
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder ClosePayment was requested but the" +
               " order is not in ORDER_ORDERED, ORDER_REFUNDABLE, or" +
               " ORDER_CLOSED state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         Vector varVector = new Vector();
         varVector.addElement(
            this.objFrameworkOrder.getMerchantName()
         );
         varVector.addElement(
            argFrameworkPayment.getBatchNumber()
         );
         varVector.addElement(
            this.objFrameworkOrder.getOrderNumber()
         );
         varVector.addElement(
            argFrameworkPayment.getTransactionNumber()
         );
         varVector.addElement(
            Integer.toString(this.objFrameworkOrder.getCurrentState())
         );
         ErrorLog.log(
            RB_ID,
            MSG_ORDER_CLOSE_PAYMENT_STATE_ERROR,
            varVector
         );
         throw new ETillCassetteException(
            ErrorLog.lookupMessage(
               RB_ID,
               MSG_ORDER_CLOSE_PAYMENT_STATE_ERROR,
               varVector
            )
         );
      }

      LdbCardPayment varCassettePayment =
         (LdbCardPayment)argFrameworkPayment.getCassetteTransaction();
      varCassettePayment.closePayment();

      if (this.objFrameworkOrder.getCurrentState() == ORDER_ORDERED) {
         this.objFrameworkOrder.setCurrentState(ORDER_REFUNDABLE);
         Supervisor.getThreadCommitPoint().addToUpdateList(
            this.objFrameworkOrder
         );
      }

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.closePayment()"
         );
      }
   }

   /********************************************************************
   * This method is used by the cassette batch when it is asked to close
   * an open batch.  It is called for every credit found in the open
   * batch.
   * <p>
   *
   * @param     argFrameworkCredit - A framework credit to be closed.
   ********************************************************************/
   public void closeCredit(
      Credit                argFrameworkCredit
   ) {
      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionEntry(
            TRACE_ID,
            "LdbCardOrder.closeCredit()"
         );
      }

      // Assertion test.  If the order is not in the correct state,
      // there is a serious defect and the cassette cannot continue.
      if (
         this.objFrameworkOrder.getCurrentState() != ORDER_REFUNDABLE &&
         this.objFrameworkOrder.getCurrentState() != ORDER_CLOSED
      ) {
         if (Trace.isAnyoneTracing()) {
            Trace.traceErrorOccurred(
               TRACE_ID,
               "LdbCardOrder CloseCredit was requested but the" +
               " order is not in ORDER_REFUNDABLE or" +
               " ORDER_CLOSED state." +
               "\n\tPRC    = " + PRC_VERB_NOT_VALID_IN_PRESENT_STATE +
               "\n\tSRC    = " + RC_ORDER +
               "\n\tState  = " + this.objFrameworkOrder.getCurrentState()
            );
         }
         Vector varVector = new Vector();
         varVector.addElement(
            this.objFrameworkOrder.getMerchantName()
         );
         varVector.addElement(
            argFrameworkCredit.getBatchNumber()
         );
         varVector.addElement(
            this.objFrameworkOrder.getOrderNumber()
         );
         varVector.addElement(
            argFrameworkCredit.getTransactionNumber()
         );
         varVector.addElement(
            Integer.toString(this.objFrameworkOrder.getCurrentState())
         );
         ErrorLog.log(
            RB_ID,
            MSG_ORDER_CLOSE_CREDIT_STATE_ERROR,
            varVector
         );
         throw new ETillCassetteException(
            ErrorLog.lookupMessage(
               RB_ID,
               MSG_ORDER_CLOSE_CREDIT_STATE_ERROR,
               varVector
            )
         );
      }

      LdbCardCredit varCassetteCredit =
         (LdbCardCredit)argFrameworkCredit.getCassetteTransaction();
      varCassetteCredit.closeCredit();

      if (Trace.isAnyoneTracing()) {
         Trace.traceFunctionExit(
            TRACE_ID,
            "LdbCardOrder.closeCredit()"
         );
      }
   }

   /********************************************************************
   * This method is used to obtain the account object that is used
   * to process transactions for this order.
   *
   * @return - The cassette account object that is used to process
   * transactions for this order.
   ********************************************************************/
   public LdbCardAccount getAccount() {
      return this.objAccount;
   }

   /********************************************************************
   * This method is used to obtain the account number that is used to
   * process transactions for this order.
   *
   * @return - The account number of the account that is used to process
   * transactions for this order.
   ********************************************************************/
   public String getAccountNumber() {
      return this.accountNumber();
   }

   /********************************************************************
   * This method is used to obtain the current approved amount.
   *
   * @return    - The current approved amount.
   ********************************************************************/
   public int    getAmount() {
      return this.objAmount;
   }

   /********************************************************************
   * This method is used to obtain the shopper's credit card number
   *
   * @return    - The shopper's credit card number.
   ********************************************************************/
   public String getPan() {
      return this.objPan;
   }

   /********************************************************************
   * This method is used to obtain the shopper's credit card number
   * when it will be displayed in a trace.
   *
   * @return   - A string with all but the first 4 and the last 4
   * characters masked.
   ********************************************************************/
   public String getObscurePan() {
      StringBuffer varBuffer = new StringBuffer(this.getPan());
      int varBegIndex = 0;
      int varEndIndex = varBuffer.length() - 5;
      for (
         int varIndex = varBegIndex;
         varIndex <= varEndIndex;
         varIndex++
      ) {
         varBuffer.setCharAt(varIndex,'*');
      }
      return varBuffer.toString();
   }

   /********************************************************************
   * This method is used to obtain the brand of the shopper's credit
   * card number.
   *
   * @return    - The brand of the shopper's credit card
   ********************************************************************/
   public String getBrand() {
      return this.objBrand;
   }

   /********************************************************************
   * This method is used to obtain the expiration data of the shopper's
   * credit card.  Note that this data is in the framework format:
   * YYYYMM.  In this case, the first two characters must be removed
   * before it can be sent to the payment engine.
   *
   * @return    - The expiration data of the shopper's credit card.
   ********************************************************************/
   public String getExpiry() {
      return this.objExpiry;
   }

   /********************************************************************
   * This method is used to obtain the shopper's street address.  In
   * this case, the payment engine only uses the street address and the
   * postal code.  Other address verifcation parameters are ignored.
   *
   * @return    - The shopper's street address.
   ********************************************************************/
   public String getAddress() {
      return this.objAddress;
   }

   /********************************************************************
   * This method is used to obtain the shopper's postal code.
   *
   * @return    - The shopper's postal code.
   ********************************************************************/
   public String getPostalCode() {
      return this.objPostalCode;
   }

   /********************************************************************
   * This method is used to find and set the cassette account object
   * that will be used to process the transactions for this order.
   * Note that the cassette account's retrieveAccount() method is used
   * to find the account object.
   *
   * @param     argMerchant - The merchant number of the account that
   * will be used to process this order.
   * @param     argAccountNumber - The account number of the account
   * that will be used to process this order.
   ********************************************************************/
   public void setAccount(
      String argMerchant,
      String argAccountNumber
   ) {
      objAccount = LdbCardAccount.retrieveAccount(
         argMerchant
      );
   }

   /********************************************************************
   * This method is used to set total approved payment amount.
   *
   * @param     argAmount - The total approved amount.
   ********************************************************************/
   public void setAmount(
      int argAmount
   ) {
      this.objAmount = argAmount;
   }

   /********************************************************************
   * This method is used to set the shopper's credit card number
   *
   * @param     argPan - The shopper's credit card number.
   ********************************************************************/
   public void setPan(
      String argPan
   ) {
      this.objPan = argPan;
   }

   /********************************************************************
   * This method is used to set the brand of the shopper's credit card.
   *
   * @param     argBrand - The brand of the shopper's credit card.
   ********************************************************************/
   public void setBrand(
      String argBrand
   ) {
      this.objBrand = argBrand;
   }

   /********************************************************************
   * This method is used to set the expiration data of the shopper's
   * credit card.  Note that the data is saved in framework
   * format:  YYYYMM.  In this case, the first two characters must be
   * removed before it can be sent to the payment engine.
   *
   * @param     argExpiry - The expiration data of the shopper's credit
   * card.
   ********************************************************************/
   public void setExpiry(
      String argExpiry
   ) {
      this.objExpiry = argExpiry;
   }

   /********************************************************************
   * This method is used to set the shopper's street address.
   *
   * @param     argAddress - The shopper's street address.
   ********************************************************************/
   public void setAddress(
      String argAddress
   ) {
      this.objAddress = argAddress;
   }

   /********************************************************************
   * This method is used to set the shopper's postal code.
   *
   * @param     argPostalCode - The shopper's postal code.
   ********************************************************************/
   public void setPostalCode(
      String argPostalCode
   ) {
      this.objPostalCode = argPostalCode;
   }
}
