/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.io.*;
import java.net.*;

import com.ibm.etill.framework.io.ETillConnection;
import com.ibm.etill.framework.io.ServerSocketComPoint;
import com.ibm.etill.framework.log.*;

/**
 * <p>ComPoints are objects that the Payment Manager framework uses to receive input
 * and to send output back out.  A ServerSocketComPoint is a specialized type of ComPoint
 * that uses a server socket to communicate.  During the start-up of Payment Manager, the
 * framework creates protocol threads for each cassette.  Each of these protocol threads has
 * a single ComPoint instance that it uses for communication.
 * <p>Pseudo-code for the protocol thread is as follows:
 * <pre>
 *  comPoint = new cassette specific ComPoint
 *  do forever
 *    eTillDataStream = comPoint.establishConnection()
 *    protocolRequest = new cassette specific protocol request created from eTillDataStream
 *    protocolResponse = new cassette specific protocol response created from eTillDataStream
 *    cassette.service(protocolRequest, protocolResponse)
 *  end do
 * </pre>
 * <p>KitCashComPoint is a simple derived class of ServerSocketComPoint.  It waits
 * for socket connections (in <code>establishConnection()</code>) and returns a KitCashConnection
 * object to the Payment Manager framework.
 *
 * @see com.ibm.etill.kitcashcassette.KitCashConnection
 */
public class KitCashComPoint extends ServerSocketComPoint {

   /**
    * Constructs a new KitCashComPoint object.
    * @param port the port on which to send and receive KitCash messages from the consumer
    */
   public KitCashComPoint(int port) {
      super(port);
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashComPoint.KitCashComPoint");
      setClassName("KitCashComPoint");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashComPoint.KitCashComPoint");
   }

   /**
    * Wait for a KitCash wallet socket connection to be estalished.
    *
    * @return a KitCashConnection object
    * @see com.ibm.etill.kitcashcassette.KitCashConnection
    */
   public ETillConnection establishConnection() throws IOException {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashComPoint.establishConnection");
      Socket socket = accept();

      // Instantiate the appropriate ETillConnection and return it
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashComPoint.establishConnection");
      return new KitCashConnection(socket);
   }
}
