/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import com.ibm.etill.framework.admin.*;
import com.ibm.etill.framework.cassette.*;
import com.ibm.etill.framework.io.*;
import com.ibm.etill.framework.log.*;
import com.ibm.etill.kitcashcassette.test.card.*;

import java.io.*;

/**
 * <p><code>KitCashPaymentRequest</code> objects encapsulate payment information
 * send from a consumer's KitCash wallet to the KitCash cassette.
 * <p>When a KitCash wallet sends a message to the KitCashComPoints, the Payment Manager framework
 * creates a new KitCashConnection object and asks the KitCash cassette to construct a
 * protocol request from it.  The framework will subsequently
 * send this request to the KitCash cassette's service(ProtocolRequest, ProtcolResponse) method
 * for processing.
 * @see com.ibm.etill.kitcashcassette.KitCashCassette#createProtocolRequest(com.ibm.etill.framework.io.ETillDataStream)
 * @see com.ibm.etill.kitcashcassette.KitCashCassette#service(com.ibm.etill.framework.cassette.CassetteRequest, com.ibm.etill.framework.cassette.CassetteResponse)
 */
public class KitCashPaymentRequest extends ProtocolRequest {

   KitCashMessageWrapper wrapper;

   /**
    * Constructs a new <code>KitCashPaymentRequest</code> object containing the necessary
    * information for the framework to be able to make a service(ProtocolRequest, ProtocolResonse)
    * call on the KitCash cassette.
    * <p>The connection object is used to read a KitCash message from the KitCash wallet.  The KitCash
    * message is contained within a KitCashMessageWrapper which wraps the message with a MIME header.
    * This KitCashMessageWrapper object can be queried to determine the merchant ID and order number
    * the message relates to as well to get the KitCash message itself.
    * @param etc the connection with the KitCash wallet
    * @see com.ibm.etill.kitcashcassette.KitCashConnection#read
    */
   public KitCashPaymentRequest(ETillConnection etc) {
      super(KitCashConstants.CASHMSG_TOKEN, etc, Admin.getCassetteAdminObject(KitCashCassette.TRACEID).getCassette());
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashPaymentRequest.KitCashPaymentRequest");
      try {
         wrapper = (KitCashMessageWrapper) etc.read();
         if (Trace.isAnyoneTracing())
            Trace.traceDebug(KitCashCassette.TRACEID,"createProtocolRequest() = " + wrapper);
      } catch (IOException ioe) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0051", ioe);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashPaymentRequest.KitCashPaymentRequest");
   }

   /**
    * Returns the cash transfer amount from the message wrapper
    */
   public int getAmount() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashPaymentRequest.getAmount");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashPaymentRequest.getAmount");
      return wrapper.getAmount();
   }

   /**
    * Returns the merchant ID associated with this KitCash protocol request
    * @return the merchant ID associated with this KitCash protocol request
    */
   public String getMerchantNumber() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashPaymentRequest.getMerchantNumber");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashPaymentRequest.getMerchantNumber");
      if (wrapper == null) return null;
      return wrapper.getReceiver();
   }

   /**
    * Returns the KitCash message associated with this KitCash protocol request
    * @return the KitCash message associated with this KitCash protocol request
    */
   public KitCashMessage getMessage() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashPaymentRequest.getMessage");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashPaymentRequest.getMessage");
      if (wrapper == null) return null;
      return wrapper.getContents();
   }

   /**
    * Returns the order number associated with this KitCash protocol request
    * @return the order number associated with this KitCash protocol request
    */
   public String getOrderNumber() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashPaymentRequest.getOrderNumber");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashPaymentRequest.getOrderNumber");
      if (wrapper == null) return null;
      return wrapper.getSenderID();
   }

   /**
    * Returns a string representation of the KitCash protocol request
    * @return a string representation of the KitCash protocol request
    */
   public String toString() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashPaymentRequest.toString");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashPaymentRequest.toString");
      return "KitCashRequest[" + getMerchantNumber() + "/" + getOrderNumber() + "/" + getMessage() + "]";
   }
}
