/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.io.*;
import java.sql.*;
import java.util.*;

import com.ibm.etill.framework.archive.*;
import com.ibm.etill.framework.cassette.*;
import com.ibm.etill.framework.log.*;
import com.ibm.etill.framework.payapi.*;
import com.ibm.etill.framework.supervisor.*;
import com.ibm.etill.kitcashcassette.test.card.*;

/**
 * <p><code>KitCashPurchase</code> objects represent purchases made by consumers.  They
 * are created by the <code>KitCashCassette</code> object whenever ReceivePayment
 * API commands are issued for the KitCash cassette.
 *
 * <p>The state of a <code>KitCashPurchase</code> object is stored in the KITCASHPURCHASE
 * database table so that they can be recovered in the event that the Payment Manager
 * shuts down mid-way through the life of the purchase object.
 *
 * <p>Purchase objects use the KitCash finite state machine to determine what actions
 * should be performed based on what inputs are received.
 *
 * <p>Since multiple payments against a single order are not permitted by the KitCash
 * payment method, <code>KitCashPurchase</code> objects hold both cassette-specific order
 * information and cassette-specific payment information.  If the payment method were to
 * allow mutiple payments, this class would probably have to be split into
 * <code>KitCashOrder</code> and <code>KitCashPayment</code> classes.
 *
 * <p>The class contains a constructor, getter and setter methods, methods to invoke
 * and respond to the KitCash finite state machine, and methods to store and
 * retrieve cassette specific information on the KITCASHPURCHASE database table.
 *
 */
public class KitCashPurchase implements CassetteOrder, CassetteTransaction, PaymentAPIConstants,
KitCashFSMUser, KitCashFSMConstants, KitCashConstants {

   private KitCashFSM fsm;
   private Order order;
   private Cassette cassette;
   private KitCashTransaction currentTransaction;
   private APIRequest apiReq;
   private APIResponse apiResp;
   private KitCashPaymentRequest kcPaymentReq;
   private ProtocolResponse protocolResp;
   private String cardNumber;
   private String accountNumber;

   private boolean purchaseAlreadyCreated;
   private boolean purchaseAlreadyUpdated;

   private static final String WAKEUP_FILE                = "KitCashWakeup";
   private static final String WAKEUP_APPLET_PARAMS_TOKEN = "$$WAKEUP_APPLET_PARAMS$$";

   /**
    * Constructs a new <code>KitCashPurchase</code> object for the KitCash cassette.
    * @param cassette a Cassette object
    * @param order the framework order
    */
   public KitCashPurchase(Cassette cassette, Order order) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.KitCashPurchase(Cassette, Order)");

      fsm = new KitCashFSM(this, Start);
      this.order = order;
      this.cassette = cassette;

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.KitCashPurchase(Cassette, Order)");
   }

   /**
    * Constructs a new <code>KitCashPurchase</code> object for the KitCash cassette.
    * @param cassette a Cassette object
    * @param order the framework order
    */
   public KitCashPurchase(Cassette cassette, Order order, ParameterTable protocolData) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.KitCashPurchase(Cassette, Order, ParameterTable)");

      fsm = new KitCashFSM(this, Start);
      this.order = order;
      this.cassette = cassette;
      accountNumber  = (String) protocolData.get(PD_ACCOUNTNUMBER);

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.KitCashPurchase(Cassette, Order, ParameterTable)");
   }

   /**
    * Returns the number of the account to be used when deposited the payment.
    * @return the account number
    */
   public String accountNumber() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.accountNumber");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.accountNumber");
      return accountNumber;
   }

   /**
    * @deprecated not required by cassette developers
    */
   public short batchRequired() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.batchRequired");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.batchRequired");
      return 0;
   }

   /**
    * Returns the Cassette instance associated with this purchase object.
    * @return the Cassette instance associated with this purchase object
    */
   public Cassette cassette() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.cassette");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.cassette");
      return cassette;
   }

   /**
    * Sends the KitCash protocol message from the consumer's wallet to the
    * merchant's KitCash card and returns the KitCash response message.
    * <p>This method is triggered in response to a protocol message being
    * receive at the KitCash comPoint.
    *
    * @return a token indicating the next action to be performed
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public Integer continuePayment() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.continuePayment");

      if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: continuePayment()");
      KitCashConnection kcc = (KitCashConnection) protocolResp.getConnection();
      KitCashMessage currentMessage = kcPaymentReq.getMessage();

      try {
         KitCashDriver driver = getAccount().getDriver();

         // send the incoming KitCash protocol message to the merchant's KitCash card
         System.out.println(currentMessage + "--------------------> merchant");
         currentMessage = currentTransaction.processMsg(currentMessage);

         // write the message back to the payer
         kcc.write(currentMessage);

         System.out.println("consumer <--------------------" + currentMessage);
      } catch (KitCashException kce) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0143", kce);
         try {kcc.writeErrorResponse(kce);} catch (IOException ioe) {}
         if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.continuePayment");
         return new Integer(ERROR_TOKEN);
      } catch (IOException ioe) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0144", ioe);
         throw new ETillAbortOperation();;
      }

      // Update the current card state on the KitCashPurchase database table
      Supervisor.getThreadCommitPoint().addToUpdateList(this);

      // This flag is used to prevent this purchase object being updated
      // more than once.  See comment in KitCashPurchase.updateRecord()
      purchaseAlreadyUpdated = false;

      Supervisor.getThreadCommitPoint().commit();

      Integer token = null;
      if ((currentMessage != null) && currentMessage.isComplete()) {
         token = new Integer(END_TOKEN);
      }

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.continuePayment");
      return token;
   }

   /**
    * Inserts a new purchase record into the KITCASHPURCHASE database table.
    * <p>The cassette should not call this method directly.  It is
    * called by the Payment Manager framework when needed.
    * @param conn the database connection to use
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public void createRecord(Connection conn) throws ETillAbortOperation {
      // When a framework order object is created (in Order.createRecord()), the cassette
      // order's createRecord()> method is automatically called to create any cassette
      // specific information.  Similarly, when a framework payment object is created (in
      // Payment.createRecord()), the cassette payment's createRecord() is called.
      // Since KitCashPurchase implements both the cassette payment and the cassette order,
      // the database table record will be created twice unless we check for previous creates.
      // The 'purchaseAlreadyCreated' flag is used for this purpose.
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.createRecord");

      if (purchaseAlreadyCreated) return;
      try {
         String sqlStmt = "INSERT INTO " + ETillArchive.geteTillDatabaseOwner() + ".KITCASHPURCHASE" +
                          " (MERCHANTNUMBER, ORDERNUMBER, CARDNUMBER, ACCOUNTNUMBER, CURRENTSTATE, CARDSTATE) VALUES" +
                          " (?,            ?,           NULL,       ?,             ?,            ?)";
         PreparedStatement pstmt = conn.prepareStatement(sqlStmt);

         int index = 1;
         pstmt.clearParameters();
         ETillArchive.addString(order.getMerchantNumber(), pstmt, index++);
         ETillArchive.addString(order.getOrderNumber(), pstmt, index++);
         ETillArchive.addString(accountNumber(), pstmt, index++);
         ETillArchive.addInteger(new Integer(fsm.getState()), pstmt, index++);
         ETillArchive.addInteger(new Integer((currentTransaction == null ? KitCashTransaction.IDLE : currentTransaction.getState())), pstmt, index++);

         if (Trace.isAnyoneTracing())
            Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: createRecord()" + KitCashCassette.lineSeparator +
                             sqlStmt + KitCashCassette.lineSeparator +
                             toString());

         pstmt.executeUpdate();
         pstmt.close();

         purchaseAlreadyCreated = true;
      } catch (SQLException e) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0102", e, order.getOrderNumber(), order.getMerchantNumber());
         throw new ETillAbortOperation();
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.createRecord");
   }

   /**
    * Deletes a purchase record from the KITCASHPURCHASE database table.
    * <p>The cassette should not call this method directly.  It is
    * called by the Payment Manager framework when needed.
    * @param conn the database connection to use
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public void deleteRecord(Connection conn) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.deleteRecord");

      try {
         String sqlDelete = "DELETE FROM " + ETillArchive.geteTillDatabaseOwner() + ".KITCASHPURCHASE " +
                            "WHERE MERCHANTNUMBER = ? " +
                            "  AND ORDERNUMBER  = ?";
         PreparedStatement pstmt = conn.prepareStatement(sqlDelete);

         int index = 1;
         pstmt.clearParameters();
         ETillArchive.addString(order.getMerchantNumber(), pstmt, index++);
         ETillArchive.addString(order.getOrderNumber(), pstmt, index++);

         if (Trace.isAnyoneTracing())
            Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: deleteRecord():" + KitCashCassette.lineSeparator +
                             sqlDelete + KitCashCassette.lineSeparator +
                             toString());

         pstmt.executeUpdate();
         pstmt.close();
      } catch (SQLException e) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0104", e, order.getOrderNumber(), order.getMerchantNumber());
         throw new ETillAbortOperation(PRC_DATABASE_ERROR, RC_NONE);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.deleteRecord");
   }

   /**
    * Returns the <code>KitCashAccount</code> object that this purchase object
    * should be deposited in.
    * @return the associated account object
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the account cannot be found
    */
   private KitCashAccount getAccount() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.getAccount");

      KitCashAccount account = KitCashAccount.getAccount(order.getMerchantNumber(), accountNumber());
      if (account == null) {
         if (Trace.isAnyoneTracing())
            Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: No account found for order=" + order.getMerchantNumber() +
                             " account=" + accountNumber());
         throw new ETillAbortOperation(PRC_UNDEFINED_OBJECT, RC_ACCOUNT);
      }

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.getAccount");
      return account;
   }

   /**
    * @deprecated not required by cassette developers
    */
   public int getFinalBatchID() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.getFinalBatchID");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.getFinalBatchID");
      return 0;
   }

   /**
    * @deprecated not required by cassette developers
    */
   public int getInitialBatchID() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.getInitialBatchID");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.getInitialBatchID");
      return 0;
   }

   /**
    * Returns the framework order associated with this cassette specific order object
    * @return the framework order
    */
   public Order getOrder() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.getOrder");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.getOrder");
      return order;
   }

   /**
    * Creates a KitCash wakeup message by reading a WAKEUP_FILE template
    * in the classpath and imbedding KitCash applet parameters into it.
    * @return a well-formed KitCash wakeup message
    */
   private String getWakeupMsg() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.getWakeupMsg");

      // look for the KitCash Wakeup message in the classpath
      StringTokenizer st = new StringTokenizer(System.getProperty("java.class.path"),
                                               System.getProperty("path.separator"));
      File file = null;
      while (st.hasMoreTokens()) {
         file = new File(st.nextToken() + System.getProperty("file.separator") + WAKEUP_FILE);
         if (file.exists()) break;
      }

      // Read in the file and insert the applet parameters where indicated
      String msg = "";
      if ((file != null) && (file.exists())) {
         try {
            if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID, "Found wakeup file as " + file.toString());
            BufferedReader in = new BufferedReader(new FileReader(file));
            String s;
            while ((s = in.readLine()) != null) {
               if (s.trim().equals(WAKEUP_APPLET_PARAMS_TOKEN)) msg += getWakeupParms();
               else if (s.startsWith("--")); // lines beginning "--" are comments
               else msg += s + KitCashCassette.lineSeparator;
            }
         } catch (IOException ioe) {
            msg = "";
         }
      } else
         if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID, "Could not find wakeup file");

      // If the wakeup file can't be found or there is a problem reading it,
      // then use a default wakeup message...
      if (msg.equals("")) {
         msg += "Content-Type: text/html" + KitCashCassette.lineSeparator + KitCashCassette.lineSeparator;
         msg += "<html><body>" + KitCashCassette.lineSeparator;
         msg += "<applet code=com.ibm.etill.kitcashcassette.test.wallet.WalletApplet.class";
         msg += " codebase=/webapp/PaymentManager/merchant archive=KitCashWallet.jar width=340 height=210>" + KitCashCassette.lineSeparator;
         msg += getWakeupParms();
         msg += "</applet>" + KitCashCassette.lineSeparator;
         msg += "</body></html>" + KitCashCassette.lineSeparator;
      }

      if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID, "Wakeup message is " + msg);
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.getWakeupMsg");
      return msg;
   }

   /**
    * Returns the KitCash wakeup applet parameters for this KitCash purchase object.
    * @return the wakeup parameters string
    */
   private String getWakeupParms() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.getWakeupParms");

      // determine which host the Payment Manager is running on...
      String server = "unknown";
      try {server = java.net.InetAddress.getLocalHost().getHostAddress();} catch (java.net.UnknownHostException e) {}

      String msg = "";
      msg += "<param name=merchanthost value='" + server + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=merchantport value='" + KitCashProfile.getProtocolPort() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=merchant value='" + order.getMerchantNumber() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=ordernumber value='" + order.getOrderNumber() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=amount value='" + order.getAmount().getAmount() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=successurl value='" + order.getsuccessURL() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=failureurl value='" + order.getfailureURL() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=cancelurl value='" + order.getcancelURL() + "'>" + KitCashCassette.lineSeparator;
      msg += "<param name=serviceurl value='" + order.getserviceURL() + "'>" + KitCashCassette.lineSeparator;

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.getWakeupParms");
      return msg;
   }

   /**
    * Updates the database to indicate that the cash for this payment has been
    * sent to the bank.
    *
    * @return a token indicating the next action to be performed
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    * @see com.ibm.etill.kitcashcassette.KitCashBatch#markOrdersAsComplete
    */
   public Integer markPaymentAsComplete() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.markPaymentAsComplete");

      Supervisor.getThreadCommitPoint().addToUpdateList(this);

      // Get the framework payment object and update its state to CLOSED.
      Payment payment = order.getUniquePayment();

      payment.setCurrentState(PAYMENT_CLOSED);
      Supervisor.getThreadCommitPoint().addToUpdateList(payment);

      // Update the order object to closed also.
      order.setCurrentState(ORDER_CLOSED);
      Supervisor.getThreadCommitPoint().addToUpdateList(order);

      // This flag is used to prevent this purchase object being updated
      // more than once. See comment in KitCashPurchase.updateRecord()
      purchaseAlreadyUpdated = false;

      Supervisor.getThreadCommitPoint().commit();

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.markPaymentAsComplete");
      // returning null indicates that there are no more operations to perform
      return null;
   }

   /**
    * Updates the database indicating that the protocol flow completed
    * successfully.
    * <p>This method is triggered in response to the merchant's KitCash
    * card indicating that the protocol flow completed successfully.
    *
    * @return a token indicating the next action to be performed
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public Integer markPaymentAsReceived() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.markPaymentAsReceived");

      // update the framework order's state to ORDERED
      order.setCurrentState(ORDER_ORDERED);
      Supervisor.getThreadCommitPoint().addToUpdateList(order);

      // get the framework payment object and update its state to APPROVED
      Payment payment = order.getPayment("1");

      payment.setCurrentState(PAYMENT_APPROVED);
      Supervisor.getThreadCommitPoint().addToUpdateList(payment);

      // This flag is used to prevent this purchase object being updated
      // more than once.  See comment in KitCashPurchase.updateRecord()
      purchaseAlreadyUpdated = false;

      Supervisor.getThreadCommitPoint().commit();

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.markPaymentAsReceived");
      // Always do a DEPOSIT after the payment has been received
      return new Integer(DEPOSIT_TOKEN);
   }

   /**
    * Updates the database to indicate that this payment has been
    * marked for deposit in the appropriate account.
    *
    * @return a token indicating the next action to be performed
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public Integer markPaymentForDeposit() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.markPaymentForDeposit");

      System.out.println(KitCashCassette.lineSeparator +
                         "KitCash cassette processing DEPOSIT command (auto deposit)");

      // update the framework order's state to ORDERED
      order.setCurrentState(ORDER_ORDERED);
      Supervisor.getThreadCommitPoint().addToUpdateList(order);

      // get the framework payment object and update its state to DEPOSITED
      Payment payment = order.getUniquePayment();
      payment.setDepositAmount(order.getAmount().getAmount());
      payment.setCurrentState(PAYMENT_DEPOSITED);
      KitCashBatch kitCashBatch = getAccount().getBatch(); // Opens a batch if one doesn't exist. Implicit Batch
      kitCashBatch.addPayment(payment);

      // This flag is used to prevent this purchase object being updated
      // more than once. See comment in KitCashPurchase.updateRecord()
      purchaseAlreadyUpdated = false;

      Supervisor.getThreadCommitPoint().commit();

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.markPaymentForDeposit");
      // returning null indicates that there are no more operations to perform
      return null;
   }

   /**
    * Updates the database to delete an order.  KitCashFSM calls this
    * when a CloseOrderRequest is received.
    *
    * @return a token (null) indicating no more operations to perform
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public Integer closeOrder() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.closeOrder");

      //make sure the order is closed.  You cannot delete an open order.
      if (order.getCurrentState() == ORDER_CLOSED) {
         Supervisor.pruneOrder(order,Supervisor.getThreadCommitPoint());
         Supervisor.getThreadCommitPoint().commit();
         apiResp.setReturnCodes(PRC_OPERATION_SUCCESS, RC_NONE);
         apiResp.setReturnMessage("Order deleted successfully.".getBytes());
      } else {
         apiResp.setReturnCodes(PRC_VERB_NOT_VALID_IN_PRESENT_STATE, RC_NONE);
         apiResp.setReturnMessage("Can not delete an order in this state.".getBytes());
      }

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.closeOrder");

      // returning null indicates that there are no more operations to perform
      return null;
   }

   /**
    * performAction; always returns an Integer(0)
    * @param int action
    * @return Integer(0)
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation if the procesing fails
    */
   public Integer performAction (int action) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.performAction");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.performAction");
      return new Integer (0);
   }

   /**
    * Processes cassette requests from the merchant.
    * <p>The KitCash Finite State Machine is used to determine which action to perform
    * based on the token embedded in the APIRequest and the <code>KitCashPurchase</code>
    * object's current state.
    * @param req the request object passed to the cassette by the Payment Manager framework
    * @param resp the response object to be used in replying to the requester
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation if the procesing fails
    */
   public void processEvent(APIRequest req, APIResponse resp) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.processEvent(API)");

      apiReq = (APIRequest) req;
      apiResp = (APIResponse) resp;

      kcPaymentReq = null;
      protocolResp = null;

      processEvent(new Integer(req.getToken()));
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.processEvent(API)");
   }

   /**
    * Processes cassette requests from the consumer's wallet.
    * <p>The KitCash Finite State Machine is used to determine which action to perform
    * based on the token (<code>CASHMSG_TOKEN</code>) and the <code>KitCashPurchase</code>
    * object's current state.
    * @param req the request object passed to the cassette by the Payment Manager framework
    * @param resp the response object to be used in replying to the requester
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation if the procesing fails
    */
   public void processEvent(KitCashPaymentRequest req, ProtocolResponse resp) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.processEvent(walletReq)");

      apiReq = null;
      apiResp = null;

      kcPaymentReq = (KitCashPaymentRequest) req;
      protocolResp = (ProtocolResponse) resp;

      // The token for KitCashPaymentRequests is always CASHMSG_TOKEN
      processEvent(new Integer(req.getToken()));

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.processEvent(walletReq)");
   }

   /**
    * Processes requests.
    * <p>The KitCash Finite State Machine is used to determine which action to perform
    * based on the input token and the <code>KitCashPurchase</code>
    * object's current state.
    * @param req the request object passed to the cassette by the PaymentManager framework
    * @param resp the response object to be used in replying to the requester
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation if the processing fails
    */
   public void processEvent(Integer token) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.processEvent(request)");

      if (Trace.isAnyoneTracing())
         Trace.traceDebug(KitCashCassette.TRACEID,KitCashCassette.lineSeparator +
                          "KitCashPurchase: processEvent() starting in state " +
                          stateStrings[fsm.getState()] + KitCashCassette.lineSeparator);

      while (token != null) {
         token = fsm.processEvent(token.intValue());
      }

      if (Trace.isAnyoneTracing())
         Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: processEvent() ending in state " +
                          stateStrings[fsm.getState()] + KitCashCassette.lineSeparator);

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.processEvent(request)");
   }

   /**
    * In case of a processing error, updates the order's state to ORDER_REJECTED and reports
    * the error to the console.
    * @return a token indicating the next action to be performed (null if no action is to be performed)
    */
   public Integer reportError() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.reportError");

      // update the framework order's state to REJECTED
      order.setCurrentState(ORDER_REJECTED);
      Supervisor.getThreadCommitPoint().addToUpdateList(order);

      // This flag is used to prevent this purchase object being updated
      // more than once.  See comment in KitCashPurchase.updateRecord()
      purchaseAlreadyUpdated = false;

      Supervisor.getThreadCommitPoint().commit();

      // Report that this purchase has finished in error state and no further processing
      // can be performed.
      ErrorLog.logError(KitCashCassette.TRACEID,"0177", null);

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.reportError");
      // returning null indicates that there are no more operations to perform
      return null;
   }

   /**
    * This method is not used.
    */
   public void resetTransaction() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.resetTransaction");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.resetTransaction");
   }

   /**
    * Retrieves the cassette specific order information from the KitCashPurchase database table.
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the database table cannot be read
    */
   public void resurrectPurchase() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.resurrectPurchase");
      String sqlQuery = "SELECT * FROM " + ETillArchive.geteTillDatabaseOwner() + ".KITCASHPURCHASE" +
                        " WHERE MERCHANTNUMBER = '" + order.getMerchantNumber() + "'" +
                        "   AND ORDERNUMBER  = '" + order.getOrderNumber() + "'";

      synchronized (ETillArchive.getArchiveLock()) {
         ETillArchive database = new ETillArchive();

         try {
            ResultSet rs = database.beginQuery(sqlQuery);

            if (rs.next()) {
               cardNumber = rs.getString("CARDNUMBER");
               accountNumber = rs.getString("ACCOUNTNUMBER").trim();
               int state = rs.getInt("CURRENTSTATE");
               fsm = new KitCashFSM(this, state);

               int cardState = rs.getInt("CARDSTATE");
               KitCashDriver driver = getAccount().getDriver();
               currentTransaction = new KitCashTransaction(driver, (byte) cardState, order.getAmount().getAmount());
            } else {
               if (Trace.isAnyoneTracing())
                  Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: retrievePurchase(): record not found for order " +
                                   order.getOrderNumber() + "and merchant " +
                                   order.getMerchantNumber());
            }
         } catch (SQLException e) {
            ErrorLog.logError(KitCashCassette.TRACEID,"0105", e, order.getOrderNumber(), order.getMerchantNumber());
            throw new ETillAbortOperation();
         } finally {
            try {database.endQuery();} catch (SQLException sqle) {ErrorLog.logError(KitCashCassette.TRACEID,"0469", sqle);}
         }
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.resurrectPurchase");
   }

   /**
    * Returns an applet tag to the merchant so that it can be sent to the
    * consumer to start up their wallet.
    * <p>When a ReceivePayment API command is received by the Payment Manager, it is sent
    * to the cassette for cassette-specific processing.  The KitCash finite state
    * machine specifies that <code>sendInitiationMsg()</code> should be called as
    * as result.
    * @return a token indicating the next action to be performed (null if no action is to be performed)
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public Integer sendInitiationMsg() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.sendInitiationMsg");

      if (Trace.isAnyoneTracing())
         Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: sendInitiationMsg()");
      System.out.println(KitCashCassette.lineSeparator +
                         "KitCash cassette received RECEIVEPAYMENT command");

      String msg = getWakeupMsg();

      apiResp.setReturnCodes(PRC_OPERATION_SUCCESS, RC_INITIALIZATION_MESSAGE);
      apiResp.setReturnMessage(msg.getBytes());

      // Create a framework payment object.
      //
      // Note: The Payment Manager framework does not perform any processing as a result of the auto-approve
      // flag being set.  It is up to the cassette to check the flag and determine what
      // to do as a result.  For KitCash, it is assumed that all RECEIVEPAYMENT commands
      // have the auto-approve flag set, so the framework payment object is created without
      // checking the flag.
      System.out.println(KitCashCassette.lineSeparator +
                         "KitCash cassette processing APPROVE command (auto approve)");

      if ((1 != order.getApproveFlag()) || !order.getDepositFlag()) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0106", null, order.getOrderNumber());
         throw new ETillAbortOperation( PRC_INVALID_PARAMETER_COMBINATION,
                                        (1 != order.getApproveFlag()) ? RC_APPROVEFLAG : RC_DEPOSITFLAG);
         // In the above case, if both auto approve & deposit Flags are not set, then error
         // is thrown on Approve flag because only after approve a capture occurs.
      }

      ParameterTable pTable = new ParameterTable();
      pTable.put(PD_ACCOUNTNUMBER, order.getAccountNumber());
      Payment payment = Supervisor.createPayment(order,
                                                 "1",     // Payment number
                                                 order.getAmount(),
                                                 pTable);

      // Sets the framework order's state to ORDER_REQUESTED and set the
      // framework payment's state to PAYMENT_APPROVED.
      // Note that the cassette is responsible for updating the state of the
      // framework order and the framework payment objects.  The cassette is also
      // responsible for defining the commit points to ensure that the database
      // is always self-consistent.
      order.setCurrentState(ORDER_RESET);
      Supervisor.getThreadCommitPoint().addToCreateList(order);

      payment.setCurrentState(PAYMENT_APPROVED);
      Supervisor.getThreadCommitPoint().addToCreateList(payment);

      // Make sure the KitCash purchase info is not created more than once.
      // Since for KitCash, there can only one payment
      // for an order, the cassette-specific payment and cassette-specific order
      // object are one and the same.  Cassette payment and order objects are
      // automatically created when their framework counterparts are created.  Since
      // KitCashPurchase objects are both, we need to make sure that we don't try
      // to create the same record twice.
      purchaseAlreadyCreated = false;

      // commit objects in their current state to the database
      Supervisor.getThreadCommitPoint().commit();

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.sendInitiationMsg");
      // returning null indicates that there are no more operations to perform
      return null;
   }

   /**
    * Sends the KitCash pay message from the consumer's wallet to the
    * merchant's KitCash card and returns the KitCash response message.
    * <p>This method is triggered in response to the first protocol message being
    * receive at the KitCash comPoint.
    *
    * @return a token indicating the next action to be performed
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public Integer startPayment() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.startPayment");

      if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: startPayment()");
      KitCashConnection kcc = (KitCashConnection) protocolResp.getConnection();
      KitCashMessage currentMessage = kcPaymentReq.getMessage();
      System.out.println(KitCashCassette.lineSeparator +
                         "KitCash protocol messages:-");
      System.out.println("-------------------------");

      try {
         KitCashDriver driver = getAccount().getDriver();

         // initialize the merchant's KitCash card and instruct it to prepare to receive money
         currentTransaction = driver.init();
         currentTransaction.receive(order.getAmount().getAmount());

         // if this is an authentication message, determine the card number
         if (currentMessage.command == KitCashMessage.AUTH_REQ)
            cardNumber = currentMessage.getID();

         // send the incoming KitCash protocol message to the merchant's KitCash card
         System.out.println(currentMessage + " --------------------> merchant");
         currentMessage = currentTransaction.processMsg(currentMessage);

         // write the message back to the payer
         kcc.write(currentMessage);

         System.out.println("consumer <-------------------- " + currentMessage);
      } catch (KitCashException kce) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0143", kce);
         try {kcc.writeErrorResponse(kce);} catch (IOException ioe) {}
         if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.startPayment");
         return new Integer(ERROR_TOKEN);
      } catch (IOException ioe) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0144", ioe);
         throw new ETillAbortOperation();
      }

      // update the framework order's state to PENDING
      order.setCurrentState(ORDER_PENDING);
      Supervisor.getThreadCommitPoint().addToUpdateList(order);

      // get the framework payment object and update its state to APPROVED
      Payment payment = order.getPayment("1");

      payment.setCurrentState(PAYMENT_PENDING);
      Supervisor.getThreadCommitPoint().addToUpdateList(payment);

      // This flag is used to prevent this purchase object being updated
      // more than once.  See comment in KitCashPurchase.updateRecord()
      purchaseAlreadyUpdated = false;

      Supervisor.getThreadCommitPoint().commit();

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.startPayment");
      // returning null indicates that there are no more operations to perform
      return null;
   }

   /**
    * Returns a string representation of the <code>KitCashPurchase</code> object
    * @return a string representation of the <code>KitCashPurchase</code> object
    */
   public String toString() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.toString");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.toString");
      return "KitCashPurchase[" + order.getMerchantNumber() + ", " +
      order.getOrderNumber() + ", " +
      fsm.getState() + ", " +
      (currentTransaction == null ? KitCashTransaction.IDLE : currentTransaction.getState()) + "]";
   }

   /**
    * Updates a purchase record on the KITCASHPURCHASE database table.
    * <p>The cassette should not call this method directly.  It is
    * called by the Payment Manager framework when needed.
    * @param conn the database connection to use
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the processing fails
    */
   public void updateRecord(Connection conn) throws ETillAbortOperation {
      // When a framework order object is updated (in Order.updateRecord()), the cassette
      // order's updateRecord() method is automatically called to update any cassette
      // specific information.  Similarly, when a framework payment object is updated (in
      // Payment.updateRecord()), the cassette payment's updateRecord() is called.
      // Since KitCashPurchase implements both the cassette payment and the cassette order,
      // the database table will be updated twice unless we check for previous updates
      // of the same information.  The 'purchaseAlreadyUpdated' flag is used for this purpose.
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashPurchase.updateRecord");

      if (purchaseAlreadyUpdated) return;
      try {
         String sqlUpdate = "UPDATE " + ETillArchive.geteTillDatabaseOwner() + ".KITCASHPURCHASE SET " +
                            "  CARDNUMBER   = ?," +
                            "  CURRENTSTATE = ?," +
                            "  CARDSTATE    = ? " +
                            "WHERE MERCHANTNUMBER = ? " +
                            "  AND ORDERNUMBER  = ?";
         PreparedStatement pstmt = conn.prepareStatement(sqlUpdate);

         pstmt.clearParameters();

         int index = 1;
         ETillArchive.addString(cardNumber, pstmt, index++);
         ETillArchive.addInteger(new Integer(fsm.getState()), pstmt, index++);
         ETillArchive.addInteger(new Integer((currentTransaction == null ? KitCashTransaction.IDLE : currentTransaction.getState())), pstmt, index++);
         ETillArchive.addString(order.getMerchantNumber(), pstmt, index++);
         ETillArchive.addString(order.getOrderNumber(), pstmt, index++);

         if (Trace.isAnyoneTracing())
            Trace.traceDebug(KitCashCassette.TRACEID,"KitCashPurchase: updateRecord():" + KitCashCassette.lineSeparator +
                             sqlUpdate + KitCashCassette.lineSeparator +
                             toString());

         pstmt.executeUpdate();
         pstmt.close();

         purchaseAlreadyUpdated = true;
      } catch (SQLException e) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0103", e, order.getOrderNumber(), order.getMerchantNumber());
         throw new ETillAbortOperation();
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashPurchase.updateRecord");
   }
}
