/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.sql.*;
import java.util.*;

import com.ibm.etill.framework.archive.*;
import com.ibm.etill.framework.cassette.query.*;
import com.ibm.etill.framework.ETill;
import com.ibm.etill.framework.log.*;
import com.ibm.etill.framework.payapi.*;
import com.ibm.etill.framework.payserverapi.*;
import com.ibm.etill.framework.supervisor.*;
import com.ibm.etill.framework.xdm.*;

/**
 * This class is used by the <code>KitCashAccountQueryRequest</code>
 * class to create KitCash specific objects as a result of a query request, and to combine those
 * objects with the framework ones supplied with the request.
 */
final public class PSServerKitCashAccount implements KitCashAccountConstants, PaymentAPIConstants {
   private String merchantNumber;
   private String accountNumber;

   private String bankHostName;
   private int bankPort;

   /**
    * Constructs a PSServerKitCashAccount object using the result set produced
    * from the query.
    * @param rs the ResultSet produced from the query
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the construction fails.
    */
   private PSServerKitCashAccount(ResultSet rs) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashAccount.PSServerKitCashAccount");
      try {
         merchantNumber = ETillArchive.readString(rs,VIEW_MERCHANTNUMBER);
         accountNumber = ETillArchive.readString(rs,VIEW_ACCOUNTNUMBER);
         bankHostName = ETillArchive.readString(rs,VIEW_BANKHOSTNAME);
         bankPort = ETillArchive.readInteger(rs,VIEW_BANKPORT).intValue();
      } catch (SQLException e) {
         if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID,"PSServerKitCashAccount(ResultSet): " + e);
         ErrorLog.logError(ETill.RB_ID, "0205", e, e.toString());
         throw new ETillAbortOperation(PRC_DATABASE_ERROR,RC_ERROR_CONNECTING_DATABASE_OR_EXEC_SQL);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashAccount.PSServerKitCashAccount");
   }

   /**
    * Combines a PSServerKitCashAccount object with a framework account object.
    * @param kitCashAccount the PSServerKitCashAccount object
    * @param account the PSServerAccount object
    */
   private static void combine(PSServerKitCashAccount kitCashAccount, PSServerAccount account) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashAccount.combine");

      //
      // Define the CassetteExtension object
      //

      CassetteExtensionObject object = new CassetteExtensionObject();

      object.addProperty(XDM_BANKHOSTNAME, kitCashAccount.bankHostName, null);
      object.addProperty(XDM_BANKPORT, new Integer(kitCashAccount.bankPort), null);

      account.setCassetteExtensionObject(object);

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashAccount.combine");
   }

   /**
    * Combines a list of PSServerKitCashAccount objects with a list of framework account objects.
    * @param kitCashAccounts the list of PSServerKitCashAccount objects
    * @param accounts the list of account objects
    */
   public static void combineQueryResults(Enumeration kitCashAccounts, Enumeration accounts) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashAccount.combineQueryResults");

      /*****************************************
      *
      *  Build a lookup table for all kitCashAccounts
      *
      *****************************************/

      Hashtable kitCashAccountTable = new Hashtable();
      while (kitCashAccounts.hasMoreElements()) {
         PSServerKitCashAccount kitCashAccount = (PSServerKitCashAccount) kitCashAccounts.nextElement();
         AccountKey key = new AccountKey(kitCashAccount.merchantNumber,kitCashAccount.accountNumber);
         kitCashAccountTable.put(key,kitCashAccount);
      }

      /*****************************************
      *
      *  Iterate through the accounts and bind them to kitCashAccounts
      *
      *****************************************/

      while (accounts.hasMoreElements()) {
         PSServerAccount account = (PSServerAccount) accounts.nextElement();
         AccountKey key = new AccountKey(account.getMerchantNumber(), account.getAccountNumber());
         PSServerKitCashAccount kitCashAccount = (PSServerKitCashAccount) kitCashAccountTable.get(key);
         if (kitCashAccount != null) combine(kitCashAccount,account);
      }

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashAccount.combineQueryResults");
   }

   /**
    * Creates PSServerKitCashAccount objects from each item in the result set.
    * @param rs the the ResultSet
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the query fails.
    */
   public static Vector query(ResultSet rs) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashAccount.query");
      Vector result = new Vector();

      try {
         while (rs.next()) {
            PSServerKitCashAccount newAccount = new PSServerKitCashAccount(rs);
            result.addElement(newAccount);
         }
      } catch (SQLException e) {
         ErrorLog.logError(ETill.RB_ID, "0205", e, e.toString());
         throw new ETillAbortOperation(PRC_DATABASE_ERROR,RC_ERROR_CONNECTING_DATABASE_OR_EXEC_SQL);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashAccount.query");
      return result;
   }
}
